{*****************************************************************************
 *                  Copyright 2008-2009 Motorola Inc.
 *                        All rights reserved
 *
 *         RFID API3 header files and defines external interface
 *			                 for RFID API3 Library
 *
 *                             rfidapi.h
 *                          rfidapiTypes.h
 *                        rfidapiConstants.h
 *                         rfidapiStructs.h
 *
 *            Pascal port by Ladislav Karrach (May 2015)
 ****************************************************************************}
unit rfidapi3;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, Classes, Windows;

type
  INT8   = shortint;
  UINT8  = byte;     PUINT8 = ^UINT8;
  INT16  = smallint;
  UINT16 = Word;     PUINT16 = ^UINT16;
  INT32  = Integer;
  UINT32 = DWord;    PUINT32 = ^UINT32;
  UINT64 = QWord;
  LPVOID = Pointer;
  PBYTEBOOL = ^BYTEBOOL; // typedef BYTE BOOLEAN

  RFID_HANDLE32 = Pointer; // void *
  PRFID_HANDLE32 = ^RFID_HANDLE32;
  RFID_HANDLE64 = UINT64;
  PRFID_HANDLE64 = ^RFID_HANDLE64;
  EVENT_HANDLE = Pointer;  // void *
  STRUCT_HANDLE = Pointer; // void *
  SOCKET_HANDLE = UINT64;

  RFID_STATUS = (
    RFID_API_SUCCESS = 0,            // Function succeeded
    RFID_API_COMMAND_TIMEOUT = 1,    // Command timeout
    RFID_API_PARAM_ERROR = 2,        // Invalid parameter given
    RFID_API_PARAM_OUT_OF_RANGE = 3, // Parameter out of range
    RFID_API_CANNOT_ALLOC_MEM = 4,   // Cannot allocate memory
    RFID_API_UNKNOWN_ERROR = 5,      // Unable to determine error
    RFID_API_INVALID_HANDLE = 6,     // Invalid Handle is provided
    RFID_API_BUFFER_TOO_SMALL = 7,   // Assigned memory buffer is small
    RFID_READER_FUNCTION_UNSUPPORTED=8, // Functionality Not Supported by Reader
    RFID_RECONNECT_FAILED = 9,       // Unable to Reconnect to reader. The Reader state might have changed. Attempt Connect API instead of Reconnect API.
    RFID_API_DATA_NOT_INITIALISED = 10, // DATA should be initalised before using the structure
    RFID_API_ZONE_ID_ALREADY_EXITS = 11,// Zone ID already exists
    RFID_API_ZONE_ID_NOT_FOUND = 12,	// Zone ID not found
    RFID_COMM_OPEN_ERROR = 100,      // Unable to open connection to the reader
    RFID_COMM_CONNECTION_ALREADY_EXISTS=101, // Connection Already exists
    RFID_COMM_RESOLVE_ERROR = 102,   // Unable to resolve IP or Host Name
    RFID_COMM_SEND_ERROR = 103,      // Error writing to transport interface
    RFID_COMM_RECV_ERROR = 104,      // Error reading from transport interface
    RFID_COMM_NO_CONNECTION = 105,   // No Connection exists to the Host
    RFID_INVALID_SOCKET = 106,       // Invalid Socket Handle passed
    RFID_READER_REGION_NOT_CONFIGURED=107, // Region of the Reader is not configured
    RFID_READER_REINITIALIZING = 108,// Reader is reinitializing after a Firmware/OEM update
    RFID_SECURE_CONNECTION_ERROR = 109, // Secure connection error. Process connStatus in SEC_CONNECTION_INFO for additional info
    RFID_ROOT_SECURITY_CERTIFICATE_ERROR = 110, // Root CA Security certificate missing or invalid
    RFID_HOST_SECURITY_CERTIFICATE_ERROR = 111, // Host Security certificate missing or invalid
    RFID_HOST_SECURITY_KEY_ERROR = 112, // Host private key missing or invalid

    RFID_CONFIG_GET_FAILED = 200,    // Unable to get the configuration
    RFID_CONFIG_SET_FAILED = 201,    // Unable to set the configuration

    RFID_CAP_NOT_SUPPORTED = 300,    // The specified capability is not supported
    RFID_CAP_GET_FAILED = 301,       // Unable to get the capability

    RFID_FILTER_NO_FILTER = 400,     // No Filter criteria specified
    RFID_FILTER_INVALID_INDEX = 401, // Invalid index specified
    RFID_FILTER_MAX_FILTERS_EXCEEDED = 402, // Reached limit for maximum number of filters that can be added
    RFID_NO_READ_TAGS = 403,         // No read Tags
    RFID_NO_REPORTED_EVENTS = 404,   // No Events have been reported from the Reader
    RFID_INVENTORY_MAX_TAGS_EXCEEDED = 405, // Tag List is full, tag cannot be stored
    RFID_INVENTORY_IN_PROGRESS = 406,// Inventory in progress, cannot perform requested operation
    RFID_NO_INVENTORY_IN_PROGRESS = 407, // There is no Inventory in progress
    RFID_TAG_LOCATING_IN_PROGRESS = 420, // Tag Locating in progress, cannot perform requested operation
    RFID_NO_TAG_LOCATING_IN_PROGRESS = 421, // There is no Tag locating operation in progress
    RFID_NXP_EAS_SCAN_IN_PROGRESS = 422, // NXP EAS SCAN Operation in progress, cannot perform requested operation
    RFID_NO_NXP_EAS_SCAN_IN_PROGRESS = 423, // There is no NXP EAS SCAN operation in progress

  	RFID_RM_INVALID_USERNAME_PASSWORD = 601,	// Invalid Username or password
  	RFID_RM_NO_UPDATION_IN_PROGRESS = 602,    // No Updation going on at present
  	RFID_RM_UPDATION_IN_PROGRESS = 603,       // Updation already in progress
  	RFID_RM_COMMAND_FAILED = 604,             // RM Command processing failed
  	RFID_INVALID_ERROR_CODE = 700             // Invalid Error-Code
  );

  ACCESS_OPERATION_STATUS = (
    ACCESS_SUCCESS = 0,		          // Tag Operation succeeded
    ACCESS_TAG_NON_SPECIFIC_ERROR = 1,	  // Non-Specific Tag Error
    ACCESS_READER_NON_SPECIFIC_ERROR = 2, // Non-Specific Reader Error
    ACCESS_NO_RESPONSE_FROM_TAG = 3,	  // No response from Tag
    ACCESS_INSUFFICENT_POWER = 4,	  // Operation Failed due to insufficient Power
    ACCESS_TAG_MEMORY_LOCKED_ERROR = 5,   // Operation Failed as Tag memory was locked
    ACCESS_TAG_MEMORY_OVERRUN_ERROR = 6,  // Operation Failed due to memory overrun
    ACCESS_ZERO_KILL_PASSWORD_ERROR = 7   // Operation Failed as Kill password provided was zero
  );

 {*
  * RFID_VERSION is a member of CONNECTION_INFO.
  *}
  RFID_VERSION = (RFID_API3_5_0 = 0, RFID_API3_5_1 = 1, RFID_API3_5_5 = 2);

 {*
  * RFID_EVENT_TYPE is a parameter for RFID_RegisterEventNotification.
  *}
  RFID_EVENT_TYPE = (
    GPI_EVENT = 0,
    TAG_DATA_EVENT = 1,
    BUFFER_FULL_WARNING_EVENT = 2,
    ANTENNA_EVENT = 3,
    INVENTORY_START_EVENT = 4,
    INVENTORY_STOP_EVENT = 5,
    ACCESS_START_EVENT = 6,
    ACCESS_STOP_EVENT = 7,
    DISCONNECTION_EVENT = 8,
    BUFFER_FULL_EVENT = 9,
    NXP_EAS_ALARM_EVENT = 10,
    READER_EXCEPTION_EVENT = 11,
    HANDHELD_TRIGGER_EVENT = 12,
    DEBUG_INFO_EVENT = 13,
    TEMPERATURE_ALARM_EVENT = 14,
    SLED_CONNECTED_EVENT = 15,
    SLED_DISCONNECTED_EVENT = 16
  );

 {*
  * READER_EXCEPTION_EVENT_TYPE specifies the event indicating the reason for Reader exception.
  *}
  READER_EXCEPTION_EVENT_TYPE = (
    UNKNOWN_EXCEPTION = 0 // Unknown exception. More data of the exception may be available in exceptionInfo of READER_EXCEPTION_EVENT_DATA.
  );

 {*
  * DISCONNECTION_EVENT_TYPE specifies the event indicating the reason for Disconnection.
  *}
  DISCONNECTION_EVENT_TYPE = (
    READER_INITIATED_DISCONNECTION = 0, // Disconnection notified from Reader
  					//  (For e.g. Disconnect initiated from the web-console).
    READER_EXCEPTION = 1,               // Reader has reported an exception.
    CONNECTION_LOST = 2                 // Connection to Reader lost, say because of a network problem. If the connection to the reader was inactive (no keep-alive
                                        //  message from the Reader) for a time greater than 10 times the timeout as specified in RFID_Connect, this event will be triggered to the application.
  );

 {*
  *  ACCESS_OPERATION_CODE specifies the Operation Code for Operation Sequence.
  *}
  ACCESS_OPERATION_CODE = (
    ACCESS_OPERATION_READ = 0,             // Read
    ACCESS_OPERATION_WRITE = 1,            // Write
    ACCESS_OPERATION_LOCK = 2,             // Lock
    ACCESS_OPERATION_KILL = 3,             // Kill
    ACCESS_OPERATION_BLOCK_WRITE = 4,      // Block Write
    ACCESS_OPERATION_BLOCK_ERASE = 5,      // Block Erase
    ACCESS_OPERATION_RECOMMISSION = 6,     // Recommission
    ACCESS_OPERATION_BLOCK_PERMALOCK = 7,  // Block Permalock
    ACCESS_OPERATION_NXP_SET_EAS = 8,      // Change EAS state of NXP tag(s)
    ACCESS_OPERATION_NXP_READ_PROTECT = 9, // Protect NXP tags from Inventory and access
    ACCESS_OPERATION_NXP_RESET_READ_PROTECT = 10,  // Remove Read Protection for NXP tags
    ACCESS_OPERATION_FUJITSU_CHANGE_WORDLOCK = 11, // Changes WordLock flags for two words in Fujitsu's 64K tags
    ACCESS_OPERATION_FUJITSU_CHANGE_BLOCKLOCK = 12,// Changes BlockLock flags for the Blocks inside a BlockGroup of Fujitsu's 64K Tags
    ACCESS_OPERATION_FUJITSU_READ_BLOCKLOCK = 13,  // Reads status of BlockLock flags inside a BlockGroup of Fujitsu's 64K Tags
    ACCESS_OPERATION_FUJITSU_BURST_WRITE = 14,     // Does a BurstWrite of Data,in multiples of 4 bytes,into Fujitsu 64K Tags and returns with number of bytes not successfully written.
    ACCESS_OPERATION_FUJITSU_BURST_ERASE = 15,     // Does a BurstErase of Data,in multiples of 4 bytes,in the 64K Fujitsu Tags and returns with number of bytes not successfully erased.
    ACCESS_OPERATION_FUJITSU_CHANGE_BLOCK_OR_AREA_GROUPPASSWORD = 16, // Changes a BlockGroup's password or an AreaGroup's password depending on whether target tag is 64K or 8K Fujitsu Tag
    ACCESS_OPERATION_FUJITSU_AREA_READLOCK = 17,  // Changes ReadLock flags for the Areas inside an AreaGroup of Fujitsu's 8K Tags
    ACCESS_OPERATION_FUJITSU_AREA_WRITELOCK = 18, // Changes WriteLock flags for the Areas inside an AreaGroup of Fujitsu's 8K Tags
    ACCESS_OPERATION_FUJITSU_AREA_WRITELOCK_WOPASSWORD = 19, // Sets WriteLock flags for the Areas inside an AreaGroup of Fujitsu's 8K Tags without requiring the AreaGroup's password
    ACCESS_OPERATION_IMPINJ_QT_WRITE = 20,   // Sets the Read/Write and Persistence flags
    ACCESS_OPERATION_IMPINJ_QT_READ = 21,    // Gets the Read/Write and Persistence flags
    ACCESS_OPERATION_NXP_CHANGE_CONFIG = 22, // NXP change config access operation
    ACCESS_OPERATION_NONE = $FF              // Not to be used for RFID_AddOperationToAccessSequence
  );

 {*
  * FORWARD_LINK_MODULATION indicates the type of forward link modulation.
  * It is a member of RF_MODE_TABLE_ENTRY.
  *}
  FORWARD_LINK_MODULATION = (
    FORWARD_LINK_MODULATION_PR_ASK = 0,
    FORWARD_LINK_MODULATION_SSB_ASK = 1,
    FORWARD_LINK_MODULATION_DSB_ASK = 2
  );

  MODULATION = (
    MV_FM0 = 0,
    MV_2 = 1,
    MV_4 = 2,
    MV_8 = 3
  );

 {*
  * DIVIDE_RATIO indicates the Divide Ratio.
  * It is a member of RF_MODE_TABLE_ENTRY.
  *}
  DIVIDE_RATIO = (
    DR_8 = 0,
    DR_64_3 = 1
  );

 {*
  * SPECTRAL_MASK_INDICATOR indicates the Spectral mask characteristics of the mode.
  * It is a member of RF_MODE_TABLE_ENTRY.
  *}
  SPECTRAL_MASK_INDICATOR = (
    SMI_UNKNOWN = 0,
    SMI_SI = 1,
    SMI_MI = 2,
    SMI_DI = 3
  );

 {*
  * COMMUNICATION_STANDARD indicates the communication standard currently set for the Reader.
  * It is a member of READER_CAPS.
  *}
  COMMUNICATION_STANDARD = (
    UNSPECIFIED=0,
    US_FCC_PART_15,
    ETSI_302_208,
    ETSI_300_220,
    AUSTRALIA_LIPD_1W,
    AUSTRALIA_LIPD_4W,
    JAPAN_ARIB_STD_T89,
    HONGKONG_OFTA_1049,
    TAIWAN_DGT_LP0002,
    KOREA_MIC_ARTICLE_5_2
  );

 {*
  * MEMORY_BANK specifies the four memory banks as supported by Class 1 Gen 2 Specification.
  *}
  MEMORY_BANK = (
    MEMORY_BANK_RESERVED = 0,	// Reserved Memory: contain the kill and access passwords
    MEMORY_BANK_EPC,		// EPC Memory: Contains CRC, Protocol-Control Bits, EPC Code
    MEMORY_BANK_TID,		// TID Memory: Contains 8-bit class identifier and identifying information
    MEMORY_BANK_USER		// User Memory: Contains user-specific data storage.
  );

 {*
  * START_TRIGGER_TYPE indicates type of start trigger.
  *}
  START_TRIGGER_TYPE = (
    START_TRIGGER_TYPE_IMMEDIATE = 0, // Start immediately when RFID_PerformInventory or RFID_PerformAccessSequence is called
    START_TRIGGER_TYPE_PERIODIC,      // Start periodically as per START_TRIGGER_PERIODIC
    START_TRIGGER_TYPE_GPI,           // Start based on GPI Trigger set by GPI_TRIGGER
    START_TRIGGER_TYPE_HANDHELD       // Start based on Handheld Gun/Button Trigger set by HANDHELD_TRIGGER
  );

 {*
  * STOP_TRIGGER_TYPE indicates the type of stop trigger.
  *}
  STOP_TRIGGER_TYPE = (
    STOP_TRIGGER_TYPE_IMMEDIATE = 0,  // Stop immediately when RFID_StopInventory or RFID_StopAccessSequence is called
    STOP_TRIGGER_TYPE_DURATION,       // Stop after a specified duration
    STOP_TRIGGER_TYPE_GPI_WITH_TIMEOUT, // Stop after a specified GPI Timeout
    STOP_TRIGGER_TYPE_TAG_OBSERVATION_WITH_TIMEOUT, // Stop after a specified number of Tags are read
    STOP_TRIGGER_TYPE_N_ATTEMPTS_WITH_TIMEOUT, // Stop after a specified number of attempts
    STOP_TRIGGER_TYPE_HANDHELD_WITH_TIMEOUT    // Stop after a specified Handheld Trigger Gun/Button Timeout
  );

 {*
  *  ANTENNA_MODE specifies the Antenna mode setting for RFID_SetAntennaMode.
  *}
  ANTENNA_MODE = (
	  ANTENNA_MODE_BISTATIC = 0,  // Bi-static  Antenna.
	  ANTENNA_MODE_MONOSTATIC = 1 // Mono-static Antenna.
  );

 {*
  *  TAG_FIELD specifies the fields to be enabled in TAG_DATA.
  *}
  TAG_FIELD = (
    ANTENNA_ID = 1,            // Fetch AntennaID on which Tag was found from Reader
    FIRST_SEEN_TIME_STAMP = 2, // Fetch Tag's First Seen Time Stamp from Reader
    LAST_SEEN_TIME_STAMP = 4,  // Fetch Tag's Last Seen Time Stamp from Reader
    PEAK_RSSI = 8,             // Fetch RSSI of the tag from Reader
    TAG_SEEN_COUNT = 16,       // Fetch Tag Seen Count of the tag from Reader
    PC = 32,                   // Fetch Tag's PC field from Reader
    XPC = 64,                  // Fetch Tag's XPC field from Reader
    CRC = 128,                 // Fetch Tag's CRC field from Reader
    CHANNEL_INDEX = 256,       // Fetch the Channel Index on which the Tag was found
    ZONE_ID = 1024,
    ZONE_NAME = 2048,
    PHASE_INFO = 4096,         // Fetch the Phase information reported by the reader when this tag was seen
    ALL_TAG_FIELDS = $1FFF     // Fetch all fields
  );

 {*
  *  TAG_EVENT specifies the event pertaining to a Tag reported via TAG_DATA.
  *}
  TAG_EVENT = (
    UNKNOWN_STATE = 0,   // This implies that the Tag is a result of autonomous mode operation and but the state of the tag is not known.
    NEW_TAG_VISIBLE = 1, // This implies that the Tag is a result of autonomous mode operation and the tag is visible for the first time.
    TAG_NOT_VISIBLE = 2, // This implies that the Tag is a result of autonomous mode operation and the tag is not visible.
    TAG_BACK_TO_VISIBILITY = 3,	// This implies that the Tag is a result of autonomous mode operation and the tag is back to visibility.
    NONE = 4             // This implies that the Tag is not a result of autonomous mode operation.
  );

 {*
  *  Report Trigger Types for Tag reporting when Reader is operating in Autonomous mode.
  *}
  TAG_EVENT_REPORT_TRIGGER = (
    NEVER = 0,	   // Disable new event reporting.
    IMMEDIATE = 1, // Report immediately when the event occurs.
    MODERATED = 2  // Report subject to a moderation time.
  );

 {*
  *  READER_TYPE specifies the Reader Type and is a parameter for RFID_Login.
  *}
  READER_TYPE = (
  	XR = 0, // All Readers of XR Series
  	FX = 1, // All Readers of FX Series
  	MC = 2  // All (LLRP compliant) Readers of MC Series
  );

 {*
  * MATCH_PATTERN specifies whether the criteria of filtering tags matching patterns A and B.
  * It is a member of POST_FILTER and ACCESS_FILTER.
  *}
  MATCH_PATTERN = (
    A_AND_B=0,     // Filter tags that match both the pattern A and pattern B.
    NOTA_AND_B,    // Filter tags that do not match pattern A where as matches pattern B.
    NOTA_AND_NOTB, // Filter tags that do not match pattern A and also do not match pattern B.
    A_AND_NOTB     // Filter Tags which match pattern A but do not match Pattern B.
  );

 {*
  * MATCH_RANGE specifies the Selection for match range.
  * It is a member of RSSI_RANGE_FILTER and TIME_RANGE_FILTER.
  *}
  MATCH_RANGE = (
    WITHIN_RANGE = 0, // Within range; between lower and upper time limits, lower and upper limit inclusive.
    OUTSIDE_RANGE = 1, // Outside range; outside lower and upper time limits, lower and upper limit inclusive.
    GREATER_THAN_LOWER_LIMIT = 2, // Greater than lower limit; Greater than lower time limit, lower limit inclusive, upper limit ignored.
    LOWER_THAN_UPPER_LIMIT = 3 // Lower than upper limit; Lower than upper time limit, upper limit inclusive, lower limit ignored.
  );

 {*
  *   OPERATION_QUALIFIER indicates the operation to be performed on the corresponding antenna
  *}
  OPERATION_QUALIFIER = (
    C1G2_OPERATION = 0, // Inventory or Access operation(Default)
    NXP_EAS_SCAN =1,	// Electronic Article Surveillance
    LOCATE_TAG = 2      // Tag locating using default algorithm. This algorithm can be performed only on Single antenna.
  );

 {*
  * HANDHELD_TRIGGER_EVENT_TYPE specifies the type trigger that occurred.
  *}
  HANDHELD_TRIGGER_EVENT_TYPE = (
    HANDHELD_TRIGGER_RELEASED = 0, // The Gun/Button on the Handheld device was released.
    HANDHELD_TRIGGER_PRESSED = 1   // The Gun/Button on the Handheld device was pressed.
  );

 {*
  *  READER_ID_TYPE indicates whether the Reader specifies the ReaderID as MAC Address or EPC-Id.
  *}
  READER_ID_TYPE = (
    MAC_ADDRESS = 0, // Reader ID: MAC Address.
    EPC_ID = 1       // Reader ID: EPC-ID.
  );

{$PACKRECORDS C}
 {**
  * SEC_CONNECTION holds additional information for establishing secure connection.
  *}
  SEC_CONNECTION_INFO = record
    secureMode: BYTEBOOL; // Indicates whether secure connection is required. Remaining parameters take effect only if secureMode is true
    validatePeerCert: BYTEBOOL; // Validate peer certificate. If true, reader certificate is validated against root certificate
    sizeCertBuff: UINT32;       // Number of bytes in clientCertBuff parameter
    clientCertBuff: PBYTE;      // LLRP client's certificate data. Non-null for client certificate authentication
    sizeKeyBuff: UINT32;        // Number of bytes in keyBuff parameter
    clientKeyBuff: PBYTE;       // Client's private key data. Non-null for client certificate authentication.
    sizePhraseBuff: UINT32;     // Number of bytes in phraseBuff parameter. Zero if key not encrypted
    phraseBuff: PBYTE;          // Private key password
    sizeRootCertBuff: UINT32;   // Number of bytes in rootCertBuff parameter
    rootCertBuff: PBYTE;        // Root certificate data. Non-null for reader certificate validation
    connStatus: ULONG;          // Status of connection. Call OpenSSL ERR_reason_error_string(connStatus) for details
    lpReserved: array[0..3] of LPVOID; // Reserved for future.
  end;
  PSEC_CONNECTION_INFO = ^SEC_CONNECTION_INFO;

 {*
  * CONNECTION_INFO holds the information related to connection.
  *}
  CONNECTION_INFO = record
  	version: RFID_VERSION;              // RFIDAPI version to be used.
	lpSecConInfo: PSEC_CONNECTION_INFO; // Secure connection parameters.
	lpReserved: array [0..4-2] of LPVOID; // Reserved for future. 1 reserved LPVOID used for PSEC_CONNECTION_INFO
  end;
  LPCONNECTION_INFO = ^CONNECTION_INFO;

 {*
  * contains struct info
  *}
  _STRUCT_INFO = record
    memoryAllocated: UINT32; //  Memory allocated by that structure.
    memoryUsed     : UINT32; //  Memory used by that structure.
  end;

 {*
  * Contains Antenna Information to be used for Inventory or Access Operations
  *}
  ANTENNA_INFO = record
    pAntennaList: ^UINT16; // Array for holding the Antennas to be used for the operation:
  			   // Range: 1 - N (N = Max. No of Antenna Supported).
    length: UINT32;        // Number of elements in array pAntennaList .

    pAntennaOpList: ^OPERATION_QUALIFIER; // Array of operations for each antenna ID specified in the pAntennaList.
                                          // Operation at nth Index of this array is meant for the antenna ID at nth Index in the pAntennaList Array.
                                          // Should be null for running a Gen2 operation on all the antenna in the pAntennaList
                                          // It is invalid to set operation as NXP_EAS_SCAN ,if this structure is being used in any Access API  or
                                          // RFID_PerformAccessSequence Call
    lpReserved: array[0..2] of LPVOID;  //  Reserved for future
  end;
  LPANTENNA_INFO = ^ANTENNA_INFO;

 {*
  * Contains the result of locating operation being performed.
  *}
  LOCATION_INFO = record
    union: record case byte of // union
      1:(relativeDistance: INT16); // Locationing result when using LOCATE_TAG mechanism.
      2:(lpReserved: array[0..15] of LPVOID); // Reserved for place holder for expansion of union.
    end;
    lpReserved1: array[0..3] of LPVOID; // Reserved for future.
  end;
  LPLOCATION_INFO = ^LOCATION_INFO;

 {*
  * Contains Impinj QT Data
  *}
  IMPINJ_QT_DATA = record
    QTControlData: UINT16;              // QT Control bits. Bit 15 controls the Short Range Feature and Bit 14 Controls the Public or Private Memory Map.
    lpReserved: array[0..3] of LPVOID;  // Reserved for future.
  end;
  LPIMPINJ_QT_DATA = ^IMPINJ_QT_DATA;

 {*
  *  Contains the result of AccessOperation on a particular tag. Applicable only when
     TagData.OpCode is one of ACCESS_OPERATION_FUJITSU_READ_BLOCKLOCK,ACCESS_OPERATION_FUJITSU_BURST_WRITE,
     ACCESS_OPERATION_FUJITSU_BURST_ERASE, ACCESS_OPERATION_IMPINJ_QT_READ, ACCESS_OPERATION_NXP_CHANGE_CONFIG
  *}
  ACCESS_OPERATION_RESULT = record
    union: record case byte of //union {
      { Result of Fujitsu's ReadBlockLock operation on a 64K Tag }
      1:(FujitsuReadBlockLockResult: record
           blockLockStatus: UINT16; // status of the 16 BlockLock flags inside the BlockGroup
         end);
      { Result of Fujitsu's BurstWrite operation on a 64K Tag }
      2:(FujitsuBurstWriteResult: record
           numberOfBytesNotWritten: UINT8;
         end);
      { Result of Fujitsu's BurstErase operation on a 64K Tag }
      3:(FujitsuBurstEraseResult: record
           numberOfBytesNotErased: UINT8;
         end);
      { Result of Impinj QT Data operation }
      4:(QtData: IMPINJ_QT_DATA);
      { Result of NXPChange config Data operation }
      5:(NXPChangeConfigWordResult: record
           ChangeConfigWord: UINT16;
         end);
      6:(lpReserved1: array[0..15] of LPVOID);
    end; //};
    lpReserved: array[0..1] of LPVOID;
  end;
  LPACCESS_OPERATION_RESULT = ^ACCESS_OPERATION_RESULT;


 {*
  * Reader ID as supported by the reader either MAC for EPC Format.
  * It is a member of READER_CAPS.
  *}
  READER_IDA = record
    typ: READER_ID_TYPE; // Reader shall support representation of ReaderID of type MAC-Address
  			                 // or EPC-ID. This field mentions the type which reader supports and
  			                 // indicates the Reader ID type of the value field in READER_ID.
    value: array[0..MAX_PATH-1] of AnsiChar; // Reader ID as reported by the Reader.
  end;
  LPREADER_IDA = ^READER_IDA;

  READER_IDW = record
    typ: READER_ID_TYPE; // Reader shall support representation of ReaderID of type MAC-Address
                         // or EPC-ID. This field mentions the type which reader supports and
                         // indicates the Reader ID type of the value field in READER_ID.
    value: array[0..MAX_PATH-1] of WCHAR; // Reader ID as reported by the Reader.
  end;
  LPREADER_IDW = ^READER_IDW;

 {*
  * RF_MODE_TABLE_ENTRY contains the various parameters of an RF Mode supported by the Reader.
  * It is a member of UHF_RF_MODE_TABLE.
  *}
  RF_MODE_TABLE_ENTRY = record
    modeIdentifer: UINT32; // Reader defined identifier as index to this RF_MODES.
    divideRatio: DIVIDE_RATIO;
    bdrValue: UINT32;
    modulation: MODULATION;
    forwardLinkModulationType: FORWARD_LINK_MODULATION;
    pieValue: UINT32;
    minTariValue: UINT32;
    maxTariValue: UINT32;
    stepTariValue: UINT32;
    spectralMaskIndicator: SPECTRAL_MASK_INDICATOR;
    epcHAGTCConformance: BYTEBOOL;
    lpReserved: array[0..4-1] of LPVOID; // Reserved for future.
  end;
  LPRF_MODE_TABLE_ENTRY = ^RF_MODE_TABLE_ENTRY;

 {*
  * UHF_RF_MODE_TABLE contains RF Mode table parameters for each air protocol.
  * It is a member of RF_MODES.
  *}
  UHF_RF_MODE_TABLE = record
    protocolID: UINT16; // Specifies the air protocol in use.
    numEntries: UINT16; // Number of entries in pTablesEntries.
    pTablesEntries: ^RF_MODE_TABLE_ENTRY; // Pointer to RF MODE Table entries.
  end;

 {*
  * RF_MODES gives information on RF Modes supported by the Reader.
  * It is a member of READER_CAPS.
  *}
  RF_MODES = record
    numTables: UINT16; // Number of RF Modes supported by the Reader.
    pUHFTables: ^UHF_RF_MODE_TABLE; // Pointer to the RF Mode tables.
  end;

 {*
  * TRANSMIT_POWER_LEVEL_TABLE gives information on the Transmit Power Levels supported by the Reader.
  * It is a member of READER_CAPS.
  *}
  TRANSMIT_POWER_LEVEL_TABLE = record
    numValues: UINT16; // Number of elements in the array pPowerValueList.
    pPowerValueList: ^UINT16; // Array of transmit power expressed in dBm*100 to allow fractional dBm representation.
  end;

 {*
  * RECEIVE_SENSITIVITY_TABLE gives information on the Receive Sensitivity values supported by the Reader.
  * It is a member of READER_CAPS.
  *}
  RECEIVE_SENSITIVITY_TABLE = record
    numValues: UINT16; // Number of elements in the array pReceiveSensitivityValueList.
    pReceiveSensitivityValueList: ^UINT32; // Array of Receive Sensitivity in dB relative to the maximum sensitivity, Possible Values: 0 to 128.
  end;

 {*
  * DUTY_CYCLE_TABLE gives information on the Duty Cycle Percentage values supported by the Reader.
  * It is a member of READER_CAPS.
  *}
  DUTY_CYCLE_TABLE = record
    numValues: UINT16; // Number of elements in the array pDutyCycleValueList. If 0, it indicates that the
                       // reader does not support Duty cycle setting.
    pDutyCycleValueList: ^UINT16; // Array of Duty Cycle values. For MC devices it is in percentage.
    lpReserved: array[0..3] of LPVOID; // Reserved for future.
  end;
  LPDUTY_CYCLE_TABLE = ^DUTY_CYCLE_TABLE;

 {*
  *  Frequency Hope Table. Zero or more instance when hopping is true.
  * It is a member of FREQ_HOP_INFO.
  *}
  FREQ_HOP_TABLE = record
    hopTableID: UINT16;
    numFreq: UINT16;
    pFreqList: ^UINT32;
  end;

 {*
  *  Frequency Hope Table Entry.
  * It is a member of READER_CAPS.
  *}
  FREQ_HOP_INFO = record
    numTables: UINT16;
    pFreqTables: ^FREQ_HOP_TABLE;
  end;

 {*
  * Fixed Frequency Table. At most one instance only when hopping is false
  * It is a member of READER_CAPS.
  *}
  FIXED_FREQ_INFO = record
    numFreq: UINT16;
    pFreqList: ^UINT32;
  end;

 {*
  * PerAntennaReceiveSensitivityRange Parameter.
  * It is a member of READER_CAPS.
  *}
  PER_ANTENNA_RECEIVE_SENSITIVITY_RANGE = record
    { Shall be defined in future when supported. }
    lpReserved: array[0..32-1] of LPVOID; // Reserved for future.
  end;

  VERSIONA = record
    moduleName: array[0..MAX_PATH-1] of AnsiChar;
    moduleVersion: array[0..MAX_PATH-1] of AnsiChar;
  end;

  VERSIONW = record
    moduleName: array[0..MAX_PATH-1] of WCHAR; // Name of module
    moduleVersion: array [0..MAX_PATH-1] of WCHAR; // Version of module
  end;
  LPVERSIONSW = ^VERSIONW;

 {*
  * READER_MODULE_VERSIONS gives versions of reader modules as reported by the Reader.
  * This is part of READER_CAPS.
  *}
  READER_MODULE_VERSIONSA = record
    numVersions: UINT16; // Number of versions in pVersionList
    pVersionList: ^VERSIONA; // Pointer to an array of VERSIONS
    lpReserved: array[0..3] of LPVOID; // Reserved  for future.
  end;
  LPREADER_MODULE_VERSIONSA = ^READER_MODULE_VERSIONSA;

  READER_MODULE_VERSIONSW = record
    numVersions: UINT16; // Number of versions in pVersionList
    pVersionList: ^VERSIONW; // Pointer to an array of VERSIONSW
    lpReserved: array[0..3] of LPVOID; // Reserved  for future.
  end;
  LPREADER_MODULE_VERSIONSW = ^READER_MODULE_VERSIONSW;

 {*
  * READER_CAPS gives information on Reader Capabilities as reported by the Reader.
  * This can be obtained by calling RFID_GetReaderCaps.
  *}
  READER_CAPSA = record
    { Sub-structure describing memory allocated and used by this structure. }
    structInfo: _STRUCT_INFO;

    readerID: READER_IDA; //  Reader ID in either MAC for EPC Format.
    { General Capabilities. }
    firmWareVersion: array[0..MAX_PATH-1] of AnsiChar; // Firmware Version of the reader
    modelName: array[0..MAX_PATH-1] of AnsiChar;       // Reader Model

    numAntennas: UINT16; // Number of Antenna Supported.
    //	AIR_PROTOCOL   airProtocolSupported;
    numGPIs: UINT16;     // Number of GPI ports.
    numGPOs: UINT16;     // Number of GPO ports.
    utcClockSupported: BYTEBOOL;
    receiveSensitivtyTable: RECEIVE_SENSITIVITY_TABLE; // Receive Sensitivty Table.
    perAntennaReceiveSensitivtyRange: PER_ANTENNA_RECEIVE_SENSITIVITY_RANGE; // Valid if Reader supports multiple antennas

    { Gen2LLRP Capabilities. }
    blockEraseSupported: BYTEBOOL; // Indicates whether Block Erase is supported by the Reader.
    blockWriteSupported: BYTEBOOL; // Indicates whether Block Write is supported by the Reader.
    stateAwareSingulationSupported: BYTEBOOL; // Indicates whether Inventory State Aware Singulation is supported by the Reader.
    recommissionSupported: BYTEBOOL; // Indicates whether Recommission is supported by the Reader.
    blockPermalockSupported: BYTEBOOL; // Indicates whether Block Permalock is supported by the Reader.
    writeUMISupported: BYTEBOOL; // Indicates whether UMI (User Memory Indicator) Write is supported by the Reader.
    radioPowerControlSupported: BYTEBOOL; // Indicates whether radio can be powered ON or OFF.
    maxNumOperationsInAccessSequence: UINT16; // Maximum Number of OpSpecs allowed per access Filter.
    maxNumPreFilters: UINT16;  // Maximum Number of Pre-Filters allowed per antenna.
    rfModes: RF_MODES; // List of RF Mode tables supported by the Reader.

    { Regulatory Capabilities. }
    countryCode: UINT16; // Country Code.
    communicationStandard: COMMUNICATION_STANDARD; // Communication Standard.

    { UHF Band capabilities. }
    transmitPowerLevelTable: TRANSMIT_POWER_LEVEL_TABLE; // Transmit Power Level Table.

    { Frequency Information. }
    hoppingEnabled: BYTEBOOL; // Is Frequency hopping enabled.
    freqHopInfo: FREQ_HOP_INFO; // Frequency Hop information.
    fixedFreqInfo: FIXED_FREQ_INFO; // Fixed Frequency Information.

    tagEventReportingSupported: BYTEBOOL; // Indicates the reader’s ability to report tag visibility state changes.
                                          //  If true, the client can choose the tag event state changes to
                                          //  monitor and report. Event state options are New Tag, Tag Invisible, or Tag Visibility Changed.
    rssiFilterSupported: BYTEBOOL;      // Indicates the reader’s ability to report tags based
                                        //  on the signal strength of the back-scattered signal from the tag.
    NXPCommandsSupported: BYTEBOOL; // Indicates whether the NXP Commands - ChangeEAS,EASAlarm,SetQuiet,ResetQuiet are supported by the Reader.
    tagLocationingSupported: BYTEBOOL; // Indicates the reader’s ability to locate a tag.
    lpDutyCycleTable: LPDUTY_CYCLE_TABLE; // List of DutyCycle percentages supported by the reader.
    FujitsuCommandsSupported: BYTEBOOL;    // Indicates whether the reader supports Fujitsu custom commands for 64K and 8K Tags are supported or not
    ImpinjCommandSupported: BYTEBOOL;      // Indicates whether the reader supports Impinj QT Commands
    lpVersions: LPREADER_MODULE_VERSIONSA; // List of versions of reader modules.
    SledBatteryStatusSupported: BYTEBOOL;  // Sled Battery Status support
    reserved1: BYTE;
    PeriodicTagReportsSupported: BYTEBOOL;  // Periodic tag reporting is supported
    TagPhaseReportingSupported: BYTEBOOL;   // Tag Phase information in tag report
    ZoneSuppported: BYTEBOOL;               // Zone support
    AntennaRFConfigSupported: BYTEBOOL;     // Radio RF capabilities - Stop condition, physical port config & SL_ALL & AB_FLIP support
    reserved32: array[0..4-1-1] of UINT32;  // for future expansions other than pointers types
    lpReserved: array[0..32-8-1] of LPVOID; // Reserved  for future.
  end;
  LPREADER_CAPSA = ^READER_CAPSA;

  READER_CAPSW = record
    { Sub-structure describing memory allocated and used by this structure. }
    structInfo: _STRUCT_INFO;
    readerID: READER_IDW;  // Reader ID in either MAC for EPC Format.
    { General Capabilities. }
    firmWareVersion: array [0..MAX_PATH-1] of WCHAR;  // Firmware Version of the reader
    modelName: array[0..MAX_PATH-1] of WCHAR;  // Reader Model
    numAntennas: UINT16;     // Number of Antenna Supported.
    // AIR_PROTOCOL   airProtocolSupported;
    numGPIs: UINT16;  // Number of GPI ports.
    numGPOs: UINT16;  // Number of GPO ports.
    utcClockSupported: BYTEBOOL;
    receiveSensitivtyTable: RECEIVE_SENSITIVITY_TABLE;  // Receive Sensitivty Table.
    perAntennaReceiveSensitivtyRange: PER_ANTENNA_RECEIVE_SENSITIVITY_RANGE;  // Valid if Reader supports multiple antennas where the antennas can have different receive sensitivity values. Not supported, may be support in future.

    { Gen2LLRP Capabilities. }
    blockEraseSupported: BYTEBOOL;  // Indicates whether Block Erase is supported by the Reader.
    blockWriteSupported: BYTEBOOL;  // Indicates whether Block Write is supported by the Reader.
    stateAwareSingulationSupported: BYTEBOOL;  // Indicates whether Inventory State Aware Singulation is supported by the Reader.
    recommissionSupported: BYTEBOOL;  // Indicates whether Recommission is supported by the Reader.
    blockPermalockSupported: BYTEBOOL;  // Indicates whether Block Permalock is supported by the Reader.
    writeUMISupported: BYTEBOOL;  // Indicates whether UMI (User Memory Indicator) Write is supported by the Reader.
    radioPowerControlSupported: BYTEBOOL; // Indicates whether radio can be powered ON or OFF.
    maxNumOperationsInAccessSequence: UINT16;  // Maximum Number of OpSpecs allowed per access Filter.
    maxNumPreFilters: UINT16;  // Maximum Number of Pre-Filters allowed per antenna.
    rfModes: RF_MODES;  // List of RF Mode tables supported by the Reader.

    { Regulatory Capabilities. }
    countryCode: UINT16;  // Country Code.
    communicationStandard: COMMUNICATION_STANDARD;  // Communication Standard.

    { UHF Band capabilities. }
    transmitPowerLevelTable: TRANSMIT_POWER_LEVEL_TABLE;  // Transmit Power Level Table.

    {  Frequency Information. }
    hoppingEnabled: BYTEBOOL;  // Is Frequency hopping enabled.
    freqHopInfo: FREQ_HOP_INFO;  // Frequency Hop information.
    fixedFreqInfo: FIXED_FREQ_INFO;  // Fixed Frequency Information.

    tagEventReportingSupported: BYTEBOOL;  // Indicates the reader’s ability to report tag visibility state changes.
                                           // If true, the client can choose the tag event state changes to
                                           // monitor and report. Event state options are New Tag, Tag Invisible, or Tag Visibility Changed.
    rssiFilterSupported: BYTEBOOL;  // Indicates the reader’s ability to report tags based
                                    // on the signal strength of the back-scattered signal from the tag.
    NXPCommandsSupported: BYTEBOOL; // Indicates whether the reader supports NXP commands like Change EAS,set Quiet, Reset Quiet,Calibrate
    tagLocationingSupported: BYTEBOOL;  // Indicates the reader’s ability to locate a tag.
    lpDutyCycleTable: LPDUTY_CYCLE_TABLE;  // List of DutyCycle percentages supported by the reader.
    FujitsuCommandsSupported: BYTEBOOL;  // Indicates the reader support for Fujitsu custom commands for 64K and 8K Tags.
    ImpinjCommandSupported: BYTEBOOL;  // Indicates the reader support for Impinj QT Commands.
    SledBatteryStatusSupported: BYTEBOOL; // Sled Battery Status support
    reserved1: BYTE;
    lpVersions: LPREADER_MODULE_VERSIONSW;  // List of versions of reader modules.
    PeriodicTagReportsSupported: BYTEBOOL;  // Periodic tag reporting is supported
    TagPhaseReportingSupported: BYTEBOOL;   // Tag Phase information in tag report
    ZoneSuppported: BYTEBOOL;               // Zone support
    AntennaRFConfigSupported: BYTEBOOL;     // Radio RF capabilities - Stop condition, physical port config & SL_ALL & AB_FLIP support
    reserved32: array[0..4-1-1] of UINT32;  // for future expansions other than pointers types
    lpReserved: array [0..32-8-1] of LPVOID;  // Reserved  for future.
  end;
  LPREADER_CAPSW = ^READER_CAPSW;

 {*
  * GPI_TRIGGER contains the GPI-Trigger information.
  * GPI based trigger can be specified for Start Trigger as well as Stop Trigger.
  *}
  GPI_TRIGGER = record
    portNumber: UINT16; //  GPI Port number on which the trigger is to be enabled.
    gpiEvent: BYTEBOOL;  //  The Boolean value that causes a GPI event to trigger.
    timeoutMilliseconds: UINT32; // Trigger timeout in milliseconds. If set to zero, it indicates there is no timeout.
                                 // Ignored for START_TRIGGER_TYPE_GPI; Valid only for STOP_TRIGGER_TYPE_GPI_WITH_TIMEOUT.
  end;
  LPGPI_TRIGGER = ^GPI_TRIGGER;

 {*
  * HANDHELD_TRIGGER contains the Trigger information for Handheld Gun/Button that can Start/Stop an operation, like Inventory, Access, TagLocationing, etc.
  * Handheld Gun/Button based trigger can be specified for Start Trigger as well as Stop Trigger.
  *}
  HANDHELD_TRIGGER = record
    handheldEvent: HANDHELD_TRIGGER_EVENT_TYPE; // The HANDHELD_TRIGGER_EVENT_TYPE that causes the operation to start.
    timeoutMilliseconds: UINT32; // Trigger timeout in milliseconds. If set to zero, it indicates there is no timeout.
  				 // Ignored for START_TRIGGER_TYPE_HANDHELD; Valid only for STOP_TRIGGER_TYPE_HANDHELD_WITH_TIMEOUT.
  end;
  LPHANDHELD_TRIGGER = ^HANDHELD_TRIGGER;

 {*
  * TRIGGER_WITH_TIMEOUT  contains information for triggers with timeout
  *}
  TRIGGER_WITH_TIMEOUT = record
    n: UINT16;                   // N attempts or N Tag-Observations.
    timeoutMilliseconds: UINT32; // For specifying a fail-safe timeout when this trigger is used as a
                                 // stop trigger. When the timeout is 0, it indicates that there is no
                                 // timeout.
  end;
  LPTRIGGER_WITH_TIMEOUT = ^TRIGGER_WITH_TIMEOUT;

 {*
  * PERIODIC_TRIGGER specifies when to start the Inventory or AccessSequece operation
  * and the time period to run the operation.
  * It is a member of START_TRIGGER.
  *}
  PERIODIC_TRIGGER = record
    periodMilliseconds: UINT32; // Non-zero value indicating the time period in milliseconds for which inventory will run.
    startTime: ^SYSTEMTIME;     // UTC Time to start the operation.
  end;
  LPPERIODIC_TRIGGER = ^PERIODIC_TRIGGER;

 {*
  * START_TRIGGER specifies the condition for starting Inventory or AccessSeqeunce operation.
  * It is a member of TRIGGER_INFO.
  *}
  START_TRIGGER = record
    typ: START_TRIGGER_TYPE; // Start trigger of type START_TRIGGER_TYPE.
    value: record case byte of // Value need not be specified for START_TRIGGER_TYPE_IMMEDIATE.
      1:(gpi: GPI_TRIGGER);           // To be specified for START_TRIGGER_TYPE_GPI.
      2:(periodic: PERIODIC_TRIGGER); // To be specified for START_TRIGGER_TYPE_PERIODIC.
      3:(handheld: HANDHELD_TRIGGER); // To be specified for START_TRIGGER_TYPE_HANDHELD.
    end;
  end;
  LPSTART_TRIGGER = ^START_TRIGGER;

 {*
  * STOP_TRIGGER specifies the condition for stopping Inventory or Access-Sequence operation.
  * It is a member of TRIGGER_INFO.
  *}
  STOP_TRIGGER = record
    typ: STOP_TRIGGER_TYPE; // Stop trigger of type STOP_TRIGGER_TYPE.
    value: record case byte of // Value corresponding to the stop trigger type.
      1:(duration: UINT32);                     // Duration in Milli-seconds if type is STOP_TRIGGER_TYPE_DURATION.
      2:(tagObservation: TRIGGER_WITH_TIMEOUT); // Upon seeing N tag observations, or timeout if type is STOP_TRIGGER_TYPE_TAG_OBSERVATION_WITH_TIMEOUT.
      3:(numAttempts: TRIGGER_WITH_TIMEOUT);    // N attempts to see all tags in the FOV(Field of View), or timeout if type is STOP_TRIGGER_TYPE_N_ATTEMPTS_WITH_TIMEOUT.
      4:(gpi: GPI_TRIGGER);                     // GPI with a timeout value if type is STOP_TRIGGER_TYPE_GPI_WITH_TIMEOUT.
      5:(handheld: HANDHELD_TRIGGER);           // Gun/Button Trigger with a timeout value if type is STOP_TRIGGER_TYPE_HANDHELD_WITH_TIMEOUT.
    end;
  end;
  LPSTOP_TRIGGER = ^STOP_TRIGGER;

 {*
  * TAG_EVENT_REPORT_INFO specifies the configuration of events for the reader to
  * report tag state changes.
  * It is a member of TRIGGER_INFO.
  *}
  TAG_EVENT_REPORT_INFO = record
    reportNewTagEvent: TAG_EVENT_REPORT_TRIGGER; // Report criteria when a new Tag is visible.
    newTagEventModeratedTimeoutMilliseconds: UINT16; // Timeout in milliseconds for moderating new tag event
                                                     // reporting. Use this only when reportNewTagEvent is set to
  						     // MODERATED.
    reportTagInvisibleEvent: TAG_EVENT_REPORT_TRIGGER; //  Report criteria when a Tag is invisible.
    tagInvisibleEventModeratedTimeoutMilliseconds: UINT16; // Timeout in milliseconds for moderating tag invisible event
                                                           // reporting. Use this only when reportTagInvisibleEvent is set to
  							   // MODERATED.
    reportTagBackToVisibilityEvent: TAG_EVENT_REPORT_TRIGGER; // Report criteria when a Tag is back to visibility.
    tagBackToVisibilityModeratedTimeoutMilliseconds: UINT16;  // Timeout in milliseconds for moderating tag back to visibility event
                                                              // reporting. Use this only when reportTagBackToVisibilityEvent is set to
  							      // MODERATED.
    lpReserved: array[0..3] of LPVOID; // Reserved for future.
  end;
  LPTAG_EVENT_REPORT_INFO = ^TAG_EVENT_REPORT_INFO;

 {**
  * REPORT TRIGGER specifies configuration for different type of reporting trigger mechanism
  * Currently supports only “periodic trigger report”
  *}
  REPORT_TRIGGERS = record
    periodicReportDuration: UINT32;  // Duration is in Seconds, Reports the tags periodically if the tag is continued to be seen after the duration has elapsed.
  				     //  0 - Report tags after the end of inventory.
  				     //  n - Report tag first time it is reader and then after every 'n' seconds (periodically) time interval if the tag is continued to be read.
  				     //  The following restrictions are applicable when using a periodic report trigger on the reader.
  				     //   1. Access Operation results will not be sent as part of the periodic report trigger and the reader sends an exception event when access operations are added with reader
                                     //   configured for periodic reporting of tags.
                                     //   2. Stop trigger of type tagObservation 'Upon_Seeing_N_Tags_Or_Timeout' is not supported with the reader configured for periodic reporting of tags.
    lpReserved: array[0..4-1] of LPVOID;
  end;
  LPREPORT_TRIGGERS = ^REPORT_TRIGGERS;

 {*
  * TRIGGER_INFO contains trigger criterea to be used for an Inventory or Access Sequence operation.
  *}
  TRIGGER_INFO = record
    startTrigger: START_TRIGGER; // Condition to be met to start the operation.
    stopTrigger: STOP_TRIGGER;   // Condition to be met to stop the operation.
    tagReportTrigger: UINT32;    // Signifies when to get triggered on TAG_READ_EVENT.
                                 //  0 - Tag reports will be triggered when the Stop-Criteria
    				 //      is met. In case of periodic trigger, reports will
    				 //      be triggerd after every period.
    				 //  n - Report when 'n' unique tags are found
    				 //  The default settings is 1, which implies to get Tag reports immediately.
                                 //  This will be ignored when lpTagEventReportInfo is not NULL.
    lpTagEventReportInfo: LPTAG_EVENT_REPORT_INFO; // Parameters for enabling and configuring events for the reader to report tag state changes.
    lpReportTriggers: LPREPORT_TRIGGERS; // specifies configuration different type of reporting trigger mechanisms.
                                         // Currently provides support for periodic report trigger and available only in FX7500.
    lpReserved: array[0..1] of LPVOID; // Reserved for future.
  end;
  LPTRIGGER_INFO = ^TRIGGER_INFO;

 {**
  * TAG_ZONE_INFO provides information about Zone based Operation.
  *}
  TAG_ZONE_INFOW = record
    zoneID: UINT16;                             // Zone Id of the Tag Events.
    zoneName: array[0..MAX_PATH-1] of WideChar; // Zone Name of the Tag Events.
    lpReserved: array[0..4-1] of LPVOID;        // Reserved for future.
  end;
  LPTAG_ZONE_INFOW = ^TAG_ZONE_INFOW;

  TAG_ZONE_INFOA = record
    zoneID: UINT16;                             // Zone Id of the Tag Events.
    zoneName: array[0..MAX_PATH-1] of AnsiChar; // Zone Name of the Tag Events.
    lpReserved: array[0..4-1] of LPVOID;        // Reserved for future.
  end;
  LPTAG_ZONE_INFOA = ^TAG_ZONE_INFOA;

 {*
  * TAG_DATA contains Tag related information.
  * When Inventory or Read Access Operation is performed, Tags reported by Reader
  * are reported back to application in the format of TAG_DATA.
  *}
  TAG_DATA = record
    { Sub-structure describing memory allocated and used by this structure.}
    structInfo: _STRUCT_INFO;

    pTagID: ^UINT8;         // Tag ID, for C1G2 this field refers EPC Data.
    tagIDLength: UINT32;    // Tag ID Length (Number of Bytes).
    tagIDAllocated: UINT32; // Memory allocated for Tag ID  (Number of Bytes) .
    PC: UINT16;             // PC BITS.
    XPC: UINT32;            // XPC BITS - XPC_W1 = LOWORD(XPC) and XPC_W2 = HIWORD(XPC) .
    CRC: UINT16;            // CRC.
    antennaID: UINT16;      // Antenna ID .
    seenTime: record case byte of {union}
      1:(upTime: record
           firstSeenTimeStamp: UINT64; // First Seen Time Stamp, Time of the first observation amongst the tag reports .
  	   lastSeenTimeStamp: UINT64;  // Last Seen Time Stamp,  Time of the last observation amongst the tag reports .
         end);
      2:(utcTime: record
           firstSeenTimeStamp: SYSTEMTIME; // First Seen Time Stamp, Time of the first observation amongst the tag reports .
  	   lastSeenTimeStamp: SYSTEMTIME;  // Last Seen Time Stamp,  Time of the last observation amongst the tag reports .
         end);
    end;
    peakRSSI: INT8;                    // PeakRSSI, Maximum RSSI value observed amongst the tag reports .
    channelIndex: UINT16;              // Channel Index, Index of the first channel the tag was seen.
    tagSeenCount: UINT16;              // Tag seen count, Number of tag reports.
    opCode: ACCESS_OPERATION_CODE;     // C1G2 Operation as a result of which this Tag is being reported. if opCode is ACCESS_OPERATION_NONE, this Tag is a result of Inventory operation.
    opStatus: ACCESS_OPERATION_STATUS; // Result of C1G2 Access Operation, opCode
    memoryBank:	MEMORY_BANK; // The type descriptor describing the Bank type. This field is to be ignored in case of opCode specified as ACCESS_OPERATION_BLOCK_PERMALOCK.
    pMemoryBankData: ^UINT8; // The data read from the specified Bank or Block Permalock Mask Data as specified by opCode.
    memoryBankDataByteOffset: UINT32; // Byte offset of the data.
    memoryBankDataLength: UINT32;     // The length of the data(Number of Bytes) .
    memoryBankDataAllocated: UINT32;  // Memory allocated(Number of Bytes).
    tagEvent: TAG_EVENT;
    tagEventTimeStamp: SYSTEMTIME; // Time Stamp at which the tagEvent occurred. This field is not relevant when tagEvent is NONE.
    lpLocation: LPLOCATION_INFO;   // location information. It will be present only when RFID_PerformTagLocationing or RFID_PerformInventory was invoked using Operation-qualifier LOCATE_TAG.
    lpAccessOperationResult: LPACCESS_OPERATION_RESULT; // Contains the result of AccessOperation on a particular tag.
    transmitPort: UINT16; // Transmit port of the Antenna or Transmit port on which the Tag read.
    receivePort: UINT16;  // Receive port of the Antenna or Transmit port on which the Tag read.
    lpZoneInfo: LPTAG_ZONE_INFOA; // Zone information on which the Tag is read.
    phaseInfo: INT16; // Phase information reported by the reader when this tag was seen.
    reserved: UINT16; // for future expansions other than pointers types
    lpReserved: array[0..6-1] of LPVOID; //  Reserved for future.
  end;
  LPTAG_DATA = ^TAG_DATA;

 {*
  * TAG_STORAGE_SETTINGS is used to specify the storage settings related to Tags
  * using RFID_SetTagStorageSettings. Maximum size of the settings are not retricted and could be
  * adjusted as per requirements/constraints of the machine (device/host) on which the
  * application is executed.
  *}
  TAG_STORAGE_SETTINGS = record
    maxTagCount: UINT32;             // Maximum Tag Count maintained in the API internal Queue.
    maxMemoryBankByteCount: UINT32;  // Maximum Size of Memory Bank in Bytes (Should be WORD aligned, i.e. double bytes).
    maxTagIDByteCount: UINT32;       // Maximum size of EPC Data in Bytes (Should be WORD aligned, i.e. double bytes).
    tagFields: UINT16;               // Bit field indicating contents of TAG_DATA.
  				     // It inturn specifies whether certain fields are required to be processed by the Reader/Dll.
  				     //  Absence of certain fields can improve performance of the Reader/Dll or result in
  				     //  specific behavior. For e.g. disabling Antenna Id can result in application
  				     //  receiving as single unique tag even though there were multiple entries of the
                                     //  same tag reported from different Antennas
    enableAccessReports: BYTEBOOL;   // Indicates whether Dll should enable reporting of
                                     //  TAG_DATA for access operations like Write, Lock, Kill, etc.
  				     //  By default, this will be false, which implies reports will be
  				     //  enabled only for Inventory, Read and Block-permalock Access operations.
    discardTagsOnInventoryStop: BYTEBOOL; //  Option to discard the tags which are reported by the reader
  				          //   when inventory/access-sequence operation has been stopped by a call
  				          //   to RFID_StopInventory or RFID_StopAccessSequence. Choosing this option
  				          //   will not purge the tags which are already queued by the Dll.
  				          //   This option will be ignored if tagReportTrigger of TRIGGER_INFO is not equal to 1.
    reserved32: UINT32;              // for future expansions other than pointers types
    lpReserved: array[0..1] of LPVOID; // Reserved for future.
  				       // 1. One DWORD consumed for enableAccessReports, tagFields, discardTagsOnInventoryStop
  end;
  LPTAG_STORAGE_SETTINGS = ^TAG_STORAGE_SETTINGS;

 {*
  *  DISCONNECTION_EVENT_DATA carries information on reason for  disconnection.
  *  This data corresponds to DISCONNECTION_EVENT.
  *}
  DISCONNECTION_EVENT_DATA = record
    eventInfo: DISCONNECTION_EVENT_TYPE; // Reason for disconnection.
    lpReserved: array[0..4-1] of LPVOID; // Reserved for future.
  end;
  LPDISCONNECTION_EVENT_DATA = ^DISCONNECTION_EVENT_DATA;

 {*
  *  READER_EXCEPTION_EVENT_DATA carries information on reason for  reader exception.
  *  This data corresponds to READER_EXCEPTION_EVENT.
  *}
  READER_EXCEPTION_EVENT_DATAW = record
    exceptionType: READER_EXCEPTION_EVENT_TYPE;  // Event indicating the reason for Reader exception.
    exceptionInfo: array[0..MAX_PATH-1] of WCHAR;  // Description of the exception as reported by the Reader.
    lpReserved: array[0..4-1] of LPVOID;  // Reserved for future.
  end;
  LPREADER_EXCEPTION_EVENT_DATAW = ^READER_EXCEPTION_EVENT_DATAW;

  READER_EXCEPTION_EVENT_DATAA = record
    exceptionType: READER_EXCEPTION_EVENT_TYPE;  // Event indicating the reason for Reader exception.
    exceptionInfo: array[0..MAX_PATH-1] of AnsiChar;  // Description of the exception as reported by the Reader.
    lpReserved: array[0..4-1] of LPVOID;  // Reserved for future.
  end;
  LPREADER_EXCEPTION_EVENT_DATAA = ^READER_EXCEPTION_EVENT_DATAA;

  READER_EXCEPTION_EVENT_DATA = READER_EXCEPTION_EVENT_DATAW;
  LPREADER_EXCEPTION_EVENT_DATA = LPREADER_EXCEPTION_EVENT_DATAW;

 {*
  * TAG_PATTERN holds Tag Pattern criteria  for Filter Operation.
  * If bit i in the mask (pTagMask) is zero, then bit i of the target
  * tag (pTagPattern)  is a don’t care (X); if bit i in the mask is one,
  * then bit i of the target tag is bit i of the tag pattern.
  * For example, “all tags” is specified using a mask length of zero.
  *}
  TAG_PATTERN = record
    memoryBank: MEMORY_BANK;  // Memory Bank to be considered for filtering.
    bitOffset: UINT16;        // Bit offset. The address of the first (msb) bit against which to apply the Tag Mask and compare with the value.
    pTagPattern: ^UINT8;      // Memory Bank Data which is to be used for comparison.
  			      // The application should allocate an UINT8 array and assign to this field.
  			      // The Memory Bank Data to be used for comparison should be
  			      // filled in the array.
    tagPatternBitCount: UINT16; // Number of bits in pTagPattern to be taken for comparison.
    pTagMask: ^UINT8;           // Bit Mask to be matched
  				// The application should allocate an UINT8 array and assign it to this field.
  				// The Tag Mask to be used for bit-wise comparison should be filled
  				// in the array.
    tagMaskBitCount: UINT16;  // Number of bits in pTagMask to be taken for comparison.
    lpReserved: array[0..3] of LPVOID; // Reserved for future.
  end;
  LPTAG_PATTERN = ^TAG_PATTERN;

 {*
  * RSSI_RANGE_FILTER is used to configure filter settings on the reader to filter tags based on the
  * received signal strength of the tag, the time the tag was read, or both.
  *}
  RSSI_RANGE_FILTER = record
    peakRSSILowerLimit: INT8;          // Peak RSSI Lower Limit
    peakRSSIUpperLimit: INT8;          // Peak RSSI Upper Limit
    matchRange: MATCH_RANGE;           // Match Range
    lpReserved: array[0..3] of LPVOID; // Reserved for future.
  end;
  LPRSSI_RANGE_FILTER = ^RSSI_RANGE_FILTER;

 {*
  * POST_FILTER holds the Tag Filter criteria  for Inventory Operation.
  * Each Tag will be checked against both pattern A and pattern B, as per the matchPattern condition.
  *}
  POST_FILTER = record
    { Sub-structure describing memory allocated and used by this structure . }
    structInfo: _STRUCT_INFO;

    lpTagPatternA: LPTAG_PATTERN; // Tag Pattern A to be used for comparison.
 				  // The application should allocate new TAG_PATTERN and assign this field to
  				  // the allocated pointer.
    lpTagPatternB: LPTAG_PATTERN; // Tag Pattern B to be used for comparison. If using only one filter,
  				  // this pointer should be assigned to NULL.
    matchPattern: MATCH_PATTERN;  // Match criteria to be used for filtering using Tag patterns A and B.

    lpRSSIRangeFilter: LPRSSI_RANGE_FILTER; // Pointer to RSSI Range Filter to be used.
    lpReserved: array[0..14] of LPVOID;     // Reserved for future.
  end;
  LPPOST_FILTER = ^POST_FILTER;

 {*
  * ACCESS_FILTER holds the Tag Filter criteria  for Access Operation.
  * Each Tag will be checked against both pattern A and pattern B, as per the matchPattern condition.
  *}
  ACCESS_FILTER = record
    { Sub-structure describing memory allocated and used by this structure . }
    structInfo: _STRUCT_INFO;

    lpTagPatternA: LPTAG_PATTERN; // Tag Pattern A to be used for comparison.
  				  // The application should allocate new TAG_PATTERN and assign this field to
  				  // the allocated pointer.
    lpTagPatternB: LPTAG_PATTERN; // Tag Pattern B to be used for comparison. If using only one filter, this pointer should be assigned to NULL.
    matchPattern: MATCH_PATTERN;  // Match criteria to be used for filtering using Tag patterns A and B.

    lpRSSIRangeFilter: LPRSSI_RANGE_FILTER; // Pointer to RSSI Range Filter to be used.
    lpReserved: array[0..16-1-1] of LPVOID; // Reserved for future.
                                            // 1 - Taken for lpRSSIRangeFilter.
  end;
  LPACCESS_FILTER = ^ACCESS_FILTER;

 {*
  * READ_ACCESS_PARAMS contains the parameters for Read Access Operation.
  *}
  READ_ACCESS_PARAMS = record
    memoryBank: MEMORY_BANK; // Memory bank from which data is to be read from.
    byteOffset: UINT16; // Byte offset; The addresss of the first byte to read from the chosen memory bank.
                        // Since Access operation is WORD aligned, offset should be in double bytes.
    byteCount: UINT16;  // Number of bytes to read.
  			// Since Access operation is WORD aligned, offset be in double bytes.
    accessPassword: UINT32; // Password to be used for the Access operation.
    lpReserved: array[0..4-1] of LPVOID; // Reserved for future.
  end;
  LPREAD_ACCESS_PARAMS = ^READ_ACCESS_PARAMS;

 {*
  *  WRITE_ACCESS_PARAMS contains the parameters for Write Access Operation.
  *}
  WRITE_ACCESS_PARAMS = record
    memoryBank: MEMORY_BANK; // Memory bank on which data is to be written to.
    byteOffset: UINT16; // Byte offset; Addresss of the first byte to be written to the chosen memory bank.
                        // Since Access operation is WORD aligned, offset be in double bytes.
    pWriteData: PUINT8; // Pointer to a UINT8 array which contains the data to be written.
  			// Write operation occurs only on word boundaries, hence it is required to
  			// input this field accordingly. For e.g The word 0x1122 will be UINT8 writeData[2] = {0x11, 0x22}.
    writeDataLength: UINT16; // Length of the array pWriteData.
    accessPassword: UINT32;  // Password to be used for the Access operation.
    lpReserved: array[0..4-1] of LPVOID; // Reserved for future.
  end;
  LPWRITE_ACCESS_PARAMS = ^WRITE_ACCESS_PARAMS;

 {*
  *  WRITE_SPECIFIC_FIELD_ACCESS_PARAMS contains the parameters for Write Access Operation.
  *}
  WRITE_SPECIFIC_FIELD_ACCESS_PARAMS = record
    pWriteData: PUINT8; // Pointer to a UINT8 array which contains the data to be written.
  			// Write operation occurs only on word boundaries, hence it is required to
  			// input this field accordingly. For e.g The word 0x1122 will be UINT8 writeData[2] = {0x11, 0x22}.
    writeDataLength: UINT16; // Length of the array pWriteData.
    accessPassword: UINT32;  // Password to be used for the Access operation.
    lpReserved: array[0..4-1] of LPVOID; // Reserved for future.
  end;
  LPWRITE_SPECIFIC_FIELD_ACCESS_PARAMS = ^WRITE_SPECIFIC_FIELD_ACCESS_PARAMS;

 {**
  *  OP_CODE_PARAMS contains the parameters for Operation Sequence.
  *}
  OP_CODE_PARAMS = record
    opCode: ACCESS_OPERATION_CODE; //  C1G2 Access Operation to be performed.
    opParams: STRUCT_HANDLE;       //  Pointer to READ_ACCESS_PARAMS or WRITE_ACCESS_PARAMS or
                                   //	KILL_ACCESS_PARAMS or LOCK_ACCESS_PARAMS or BLOCK_ERASE_ACCESS_PARAMS.
    lpReserved: array[0..4-1] of LPVOID; // Reserved for future.
  end;
  LPOP_CODE_PARAMS = ^OP_CODE_PARAMS;

 {*
  * VERSION_INFO contains RFID Version Info
  *}
  VERSION_INFOA = record
    dllVersion: array[0..MAX_PATH-1] of AnsiChar; // Specifies the version of RFIDAPI3 C Dll.
  end;
  LPVERSION_INFOA = ^VERSION_INFOA;

 {*
  * LOGIN_INFO contains the Login information for logging into the reader.
  *}
  LOGIN_INFOA = record
  	hostName: array[0..MAX_PATH-1] of AnsiChar;   // Reader IP or Reader Host Name
  	userName: array[0..MAX_PATH-1] of AnsiChar;
  	password: array[0..MAX_PATH-1] of AnsiChar;
  	version: RFID_VERSION;                        // RFIDAPI version to be used
  	forceLogin: BYTEBOOL;                         // Force Login this host. This setting is valid only for Merlin version 1.1.0 onwards.
                                                  // To work with FX readers of version lesser than 1.1.0, which doesn't
                                                  // support Forcelogin feature, give this value as false.
  	reserved1: array[0..3-1] of BYTE;             // 3 bytes left after using forceLogin
  	lpReserved: array[0..4-2-1] of LPVOID;        // Reserved for future.
                                                  //  DWORD 1 - Used for version,
                                                  //  DWORD 2 - forceLogin
  end;
  LPLOGIN_INFOA = ^LOGIN_INFOA;

 {*
  * READER_INFO gives information on the Reader's name, location, description and contact.
  *}
  READER_INFOA = record
  	name: array[0..MAX_PATH-1] of AnsiChar;
  	description: array[0..MAX_PATH-1] of AnsiChar;
  	location: array[0..MAX_PATH-1] of AnsiChar;
  	contact: array[0..MAX_PATH-1] of AnsiChar;
  	lpReserved: array[0..4-1] of LPVOID;    //  Reserved for future.
  end;
  LPREADER_INFOA = ^READER_INFOA;

 {*
  * ERROR_INFO provides additional information on the last API that failed.
  *}
  ERROR_INFOA = record
    errorTimeStamp: SYSTEMTIME;  // UTC Timestamp at which the error occurred.
    rfidStatusCode: RFID_STATUS; // Error code of the operation that failed.
    statusDesc: array[0..MAX_PATH-1] of AnsiChar; // Error description pertaining to rfidStatusCode.
    vendorMessage: array[0..MAX_PATH-1] of AnsiChar; // Vendor specific error message for the error that occurred; E.g. LLRP message or Reader Management Message.
  end;
  LPERROR_INFOA = ^ERROR_INFOA;

var
  // WINAPI = stdcall
  RFID_ConnectA: function(
      pReaderHandle: PRFID_HANDLE32;
  		pHostName: PAnsiChar;
  		port: UINT32;
  		timeoutMilliseconds: UINT32;
  		lpConnectionInfo: LPCONNECTION_INFO): RFID_STATUS; stdcall;
  RFID_Reconnect: function(
      readerHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  RFID_Disconnect: function(
  		readerHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  // Configuration
  RFID_GetPhysicalAntennaProperties: function(
      readerHandle: RFID_HANDLE32;
      antennaID: UINT16;
      pStatus: PBYTEBOOL;
      pAntennaGain: PUINT32): RFID_STATUS; stdcall;
  RFID_GetAntennaConfig: function(
      readerHandle: RFID_HANDLE32;
      antennaID: UINT16;
      pReceiveSensitivityIndex: PUINT16;
      pTransmitPowerIndex: PUINT16;
      pTransmitFrequencyIndex: PUINT16): RFID_STATUS; stdcall;
  RFID_SetAntennaConfig: function(
      readerHandle: RFID_HANDLE32;
      antennaID: UINT16;
      pReceiveSensitivityIndex: UINT16;
      pTransmitPowerIndex: UINT16;
      pTransmitFrequencyIndex: UINT16): RFID_STATUS; stdcall;
  // Capabilities
  RFID_GetDllVersionInfoA: function(
      lpVersionInfo: LPVERSION_INFOA): RFID_STATUS; stdcall;
  RFID_GetReaderCapsA: function(
      readerHandle: RFID_HANDLE32;
      lpReaderCaps: LPREADER_CAPSA): RFID_STATUS; stdcall;
  // Helper functions
  RFID_GetErrorDescriptionA: function(
		  errorCode: RFID_STATUS): PAnsiChar; stdcall;
  RFID_GetLastErrorInfoA: function(
      readerHandle: RFID_HANDLE32;
      lpErrorInfo: LPERROR_INFOA): RFID_STATUS; stdcall;
  // Event Management
  RFID_RegisterEventNotification: function(
      readerHandle: RFID_HANDLE32;
      eventType: RFID_EVENT_TYPE;
      notifyObject: EVENT_HANDLE): RFID_STATUS; stdcall;
  RFID_DeregisterEventNotification: function(
      readerHandle: RFID_HANDLE32;
      eventType: RFID_EVENT_TYPE): RFID_STATUS; stdcall;
  RFID_GetEventData: function(
      readerHandle: RFID_HANDLE32;
      eventType: RFID_EVENT_TYPE;
      pEventData: STRUCT_HANDLE): RFID_STATUS; stdcall;
  // Inventory
  RFID_PerformInventory: function(
      readerHandle: RFID_HANDLE32;
      lpPostFilter: LPPOST_FILTER;
      lpAntennaInfo: LPANTENNA_INFO;
      lpTriggerInfo: LPTRIGGER_INFO;
      rsvd: LPVOID): RFID_STATUS; stdcall;
  RFID_StopInventory: function(
      readerHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  // Tag Management
  RFID_GetTagStorageSettings: function(
      readerHandle: RFID_HANDLE32;
      lpTagStorageSettings: LPTAG_STORAGE_SETTINGS): RFID_STATUS; stdcall;
  RFID_SetTagStorageSettings: function(
  		readerHandle: RFID_HANDLE32;
  		lpTagStorageSettings: LPTAG_STORAGE_SETTINGS): RFID_STATUS; stdcall;
  RFID_AllocateTag: function(
      readerHandle: RFID_HANDLE32):LPTAG_DATA; stdcall;
  RFID_DeallocateTag: function(
      readerHandle: RFID_HANDLE32;
      lpTagData: LPTAG_DATA): RFID_STATUS; stdcall;
  RFID_GetReadTag: function(
      readerHandle: RFID_HANDLE32;
      lpTagData: LPTAG_DATA): RFID_STATUS; stdcall;
  RFID_PurgeTags: function(
      readerHandle: RFID_HANDLE32;
      rsvd: LPVOID): RFID_STATUS; stdcall;
  // Access
  RFID_Read: function(
      readerHandle: RFID_HANDLE32;
      pTagID: PUINT8;
      tagIDLength: UINT32;
      lpReadAccessParams: LPREAD_ACCESS_PARAMS;
      lpAccessFilter: LPACCESS_FILTER;
      lpAntennaInfo: LPANTENNA_INFO;
      lpTagData: LPTAG_DATA;
      rsvd: LPVOID): RFID_STATUS; stdcall;
  RFID_Write: function(
    	readerHandle: RFID_HANDLE32;
    	pTagID: PUINT8;
      tagIDLength: UINT32;
    	lpWriteAccessParams: LPWRITE_ACCESS_PARAMS;
    	lpAccessFilter: LPACCESS_FILTER;
    	lpAntennaInfo: LPANTENNA_INFO;
    	rsvd: LPVOID): RFID_STATUS; stdcall;
  RFID_WriteTagID: function(
    	readerHandle: RFID_HANDLE32;
    	pTagID: PUINT8;
      tagIDLength: UINT32;
    	lpWriteSpecificFieldAccessParams: LPWRITE_SPECIFIC_FIELD_ACCESS_PARAMS;
    	lpAntennaInfo: LPANTENNA_INFO;
    	rsvd: LPVOID): RFID_STATUS; stdcall;
  // Access sequence
  RFID_InitializeAccessSequence: function(
  		readerHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  RFID_AddOperationToAccessSequence: function(
  		readerHandle: RFID_HANDLE32;
  		lpOpCodeParams: LPOP_CODE_PARAMS;
  		pOpCodeIndex: PUINT32): RFID_STATUS; stdcall;
  RFID_DeleteOperationFromAccessSequence: function(
  		readerHandle: RFID_HANDLE32;
  		opCodeIndex: UINT32): RFID_STATUS; stdcall;
  RFID_DeinitializeAccessSequence: function(
  		readerHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  RFID_PerformAccessSequence: function(
  		readerHandle: RFID_HANDLE32;
  		lpAccessFilter: LPACCESS_FILTER;
  		lpAntennaInfo: LPANTENNA_INFO;
  		lpTriggerInfo: LPTRIGGER_INFO;
  		rsvd: LPVOID): RFID_STATUS; stdcall;
  RFID_StopAccessSequence: function(
  		readerHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  RFID_GetLastAccessResult: function(
  		readerHandle: RFID_HANDLE32;
  		pSuccessCount: PUINT32;
  		pFailureCount: PUINT32): RFID_STATUS; stdcall;
  // Reader management
  RFID_LoginA: function(
		pReaderManagementHandle: RFID_HANDLE32;
		lpLoginInfo: LPLOGIN_INFOA;
		readerType: READER_TYPE;
		secureMode: BYTEBOOL;
		rsvd: LPVOID): RFID_STATUS; stdcall;
  RFID_Logout: function(
		readerManagementHandle: RFID_HANDLE32): RFID_STATUS; stdcall;
  RFID_EnableReadPoint: function(
		readerManagementHandle: RFID_HANDLE32;
		antennaID: UINT16;
		status: BYTEBOOL): RFID_STATUS; stdcall;
  RFID_GetAntennaMode: function(
      readerManagementHandle: RFID_HANDLE32;
      var AntennaMode: ANTENNA_MODE): RFID_STATUS; stdcall;
  RFID_SetAntennaMode: function(
      readerManagementHandle: RFID_HANDLE32;
      antennaMode: ANTENNA_MODE): RFID_STATUS; stdcall;
  RFID_GetReaderInfoA: function(
      readerManagementHandle: RFID_HANDLE32;
      lpReaderInfo: LPREADER_INFOA): RFID_STATUS; stdcall;

procedure InitializeRfidApi;
procedure ReleaseRfidApi;

type

  TRFIDReader = class;

  { TRFIDReaderThread }

  TRFIDReaderThread = class(TThread)
    private
      FrfidReader: TRFIDReader;
      FpTagData: LPTAG_DATA;
      FdisconnectionEventData: DISCONNECTION_EVENT_DATA;
      FreaderExceptionEventData: READER_EXCEPTION_EVENT_DATA;
      procedure DoOnReadTag;
      procedure DoOnInventoryStop;
      procedure DoOnDisconnect;
      procedure DoOnException;
      procedure DoOnTimeout;
    public
      constructor Create(ARFIDReader: TRFIDReader);
      procedure Execute; override;
  end;

  { TRFIDReader }

  TOnReadTagEvent = procedure(Sender: TRFIDReader; lpTagData: LPTAG_DATA) of object;
  TOnReadTagEventCallback = procedure(Sender: TRFIDReader; lpTagData: LPTAG_DATA);
  TOnInventoryStopEvent = procedure(Sender: TRFIDReader) of object;
  TOnDisconnectEvent = procedure(Sender: TRFIDReader; lpDisconnectionData: LPDISCONNECTION_EVENT_DATA) of object;
  TOnExceptionEvent = procedure(Sender: TRFIDReader; lpReaderExceptionData: LPREADER_EXCEPTION_EVENT_DATA) of object;

  TRFIDReader = class
    private
      FHandle: RFID_HANDLE32;
      FHostName: string;
      FReaderCaps: READER_CAPSA;
      FAutonomousMode: boolean;
      FTagList: TStringList; // unique list of seen tag id's

      FOnReadTag: TOnReadTagEvent; // object type
      FOnReadTagCallback: TOnReadTagEventCallback; // procedural type
      FOnInventoryStop: TOnInventoryStopEvent;
      FOnDisconnect: TOnDisconnectEvent;
      FOnException: TOnExceptionEvent;
      FOnIdle: TOnInventoryStopEvent;

      FRfidReaderThread: TRFIDReaderThread;
      FTagDataEvent: HANDLE;

      procedure CheckStatus(Status: RFID_STATUS);
      procedure SetTriggerInfo(out triggerInfo: TRIGGER_INFO; out tagEventReportInfo: TAG_EVENT_REPORT_INFO);
    public
      constructor Create(AHostName: string = '');
      destructor Destroy; override;
      procedure Connect;
      procedure Disconnect;
      function HasAntenna(antennaID: UINT16): boolean;
      procedure SetAntennaConfig(const antennaIDs: array of UINT16; ReceiveSensitivityIndex: UINT16; TransmitPowerIndex: UINT16; TransmitFrequencyIndex: UINT16);
      procedure RegisterEventNotification(eventType: RFID_EVENT_TYPE; notifyObject: EVENT_HANDLE);
      procedure DeregisterEventNotification(eventType: RFID_EVENT_TYPE);
      function GetTagStorageSettings: TAG_STORAGE_SETTINGS;
      procedure SetTagStorageSettings(lpTagStorageSettings: LPTAG_STORAGE_SETTINGS);
      function AllocateTag: LPTAG_DATA;
      procedure DeallocateTag(lpTagData: LPTAG_DATA);
      procedure PurgeTags;
      procedure PerformInventory; overload;
      procedure PerformInventory(lpTriggerInfo: LPTRIGGER_INFO); overload;
      procedure StopInventory;
      procedure InitializeAccessSequence;
      function AddOperationToAccessSequence(lpOpCodeParams: LPOP_CODE_PARAMS): integer;
      procedure DeinitializeAccessSequence;
      procedure PerformAccessSequence; overload;
      procedure PerformAccessSequence(lpTriggerInfo: LPTRIGGER_INFO); overload;
      procedure StopAccessSequence;
      function GetEventData(eventType: RFID_EVENT_TYPE; pEventData: STRUCT_HANDLE): boolean;
      function GetReadTag(lpTagData: LPTAG_DATA): boolean;
      function Read(lpTagData_in, lpTagData_out: LPTAG_DATA): boolean;
      procedure WriteTagId(lpTagData: LPTAG_DATA; const TagId: integer); overload;
      procedure WriteTagId(lpTagData: LPTAG_DATA; const TagId: AnsiString); overload;
      class function IsTagIdA(lpTagData: LPTAG_DATA): boolean;     // Is Ascii ?
      class function GetTagIdA(lpTagData: LPTAG_DATA): AnsiString; // Ascii
      class function GetTagIdH(lpTagData: LPTAG_DATA): AnsiString; // Hexadecimal
      function GetLastErrorInfo: ERROR_INFOA;
      property Handle: RFID_HANDLE32 read FHandle;
      property ReaderCaps: READER_CAPSA read FReaderCaps;
      property TagList: TStringList read FTagList;
      property OnReadTagCallback: TOnReadTagEventCallback read FOnReadTagCallback write FOnReadTagCallback;
    published
      property AutonomousMode: boolean read FAutonomousMode write FAutonomousMode;
      property HostName: string read FHostName write FHostName;
      property OnReadTag: TOnReadTagEvent read FOnReadTag write FOnReadTag;
      property OnInventoryStop: TOnInventoryStopEvent read FOnInventoryStop write FOnInventoryStop;
      property OnDisconnect: TOnDisconnectEvent read FOnDisconnect write FOnDisconnect;
      property OnException: TOnExceptionEvent read FOnException write FOnException;
      property OnIdle: TOnInventoryStopEvent read FOnIdle write FOnIdle;
  end;

  ERFIDError = class(Exception);

implementation

var
  RfidLibraryHandle: THandle;
  RfidRefCount: integer;

procedure InitializeRfidApi;
const LibraryName = 'RFIDAPI32PC.DLL';
begin
   inc(RfidRefCount);
   if RfidRefCount = 1 then
   begin
     RfidLibraryHandle := LoadLibrary(LibraryName);
     if RfidLibraryHandle = 0 then
     begin
       RfidRefCount := 0;
       raise EInOutError.CreateFmt('Can not load Rfid3 library "%s" (%s)',
                                   [LibraryName, SysErrorMessage(GetLastOSError)]);
     end;

     pointer(RFID_ConnectA) := GetProcAddress(RfidLibraryHandle,'RFID_ConnectA');
     pointer(RFID_Disconnect) := GetProcAddress(RfidLibraryHandle,'RFID_Disconnect');
     pointer(RFID_GetDllVersionInfoA) := GetProcAddress(RfidLibraryHandle,'RFID_GetDllVersionInfoA');
     pointer(RFID_GetReaderCapsA) := GetProcAddress(RfidLibraryHandle,'RFID_GetReaderCapsA');
     pointer(RFID_GetErrorDescriptionA) := GetProcAddress(RfidLibraryHandle,'RFID_GetErrorDescriptionA');
     pointer(RFID_GetLastErrorInfoA) := GetProcAddress(RfidLibraryHandle,'RFID_GetLastErrorInfoA');
     // Configuration
     pointer(RFID_GetPhysicalAntennaProperties) := GetProcAddress(RfidLibraryHandle,'RFID_GetPhysicalAntennaProperties');
     pointer(RFID_GetAntennaConfig) := GetProcAddress(RfidLibraryHandle,'RFID_GetAntennaConfig');
     pointer(RFID_SetAntennaConfig) := GetProcAddress(RfidLibraryHandle,'RFID_SetAntennaConfig');
     // Event Management
     pointer(RFID_RegisterEventNotification) := GetProcAddress(RfidLibraryHandle,'RFID_RegisterEventNotification');
     pointer(RFID_DeregisterEventNotification) := GetProcAddress(RfidLibraryHandle,'RFID_DeregisterEventNotification');
     pointer(RFID_GetEventData) := GetProcAddress(RfidLibraryHandle,'RFID_GetEventData');
     // Inventory
     pointer(RFID_PerformInventory) := GetProcAddress(RfidLibraryHandle,'RFID_PerformInventory');
     pointer(RFID_StopInventory) := GetProcAddress(RfidLibraryHandle,'RFID_StopInventory');
     // Tag Management
     pointer(RFID_GetTagStorageSettings) := GetProcAddress(RfidLibraryHandle,'RFID_GetTagStorageSettings');
     pointer(RFID_SetTagStorageSettings) := GetProcAddress(RfidLibraryHandle,'RFID_SetTagStorageSettings');
     pointer(RFID_AllocateTag) := GetProcAddress(RfidLibraryHandle,'RFID_AllocateTag');
     pointer(RFID_DeallocateTag) := GetProcAddress(RfidLibraryHandle,'RFID_DeallocateTag');
     pointer(RFID_GetReadTag) := GetProcAddress(RfidLibraryHandle,'RFID_GetReadTag');
     pointer(RFID_PurgeTags) := GetProcAddress(RfidLibraryHandle,'RFID_PurgeTags');
     // Read
     pointer(RFID_Read) := GetProcAddress(RfidLibraryHandle,'RFID_Read');
     pointer(RFID_Write) := GetProcAddress(RfidLibraryHandle,'RFID_Write');
     pointer(RFID_WriteTagID) := GetProcAddress(RfidLibraryHandle,'RFID_WriteTagID');
     // Access sequence
     pointer(RFID_InitializeAccessSequence) := GetProcAddress(RfidLibraryHandle,'RFID_InitializeAccessSequence');
     pointer(RFID_AddOperationToAccessSequence) := GetProcAddress(RfidLibraryHandle,'RFID_AddOperationToAccessSequence');
     pointer(RFID_DeleteOperationFromAccessSequence) := GetProcAddress(RfidLibraryHandle,'RFID_DeleteOperationFromAccessSequence');
     pointer(RFID_DeinitializeAccessSequence) := GetProcAddress(RfidLibraryHandle,'RFID_DeinitializeAccessSequence');
     pointer(RFID_PerformAccessSequence) := GetProcAddress(RfidLibraryHandle,'RFID_PerformAccessSequence');
     pointer(RFID_StopAccessSequence) := GetProcAddress(RfidLibraryHandle,'RFID_StopAccessSequence');
     pointer(RFID_GetLastAccessResult) := GetProcAddress(RfidLibraryHandle,'RFID_GetLastAccessResult');
     // Reader management
     pointer(RFID_LoginA) := GetProcAddress(RfidLibraryHandle,'RFID_LoginA');
     pointer(RFID_Logout) := GetProcAddress(RfidLibraryHandle,'RFID_Logout');
     pointer(RFID_GetAntennaMode) := GetProcAddress(RfidLibraryHandle,'RFID_GetAntennaMode');
     pointer(RFID_GetReaderInfoA) := GetProcAddress(RfidLibraryHandle,'RFID_GetReaderInfoA');
   end;
end;

procedure ReleaseRfidApi;
begin
  if RfidRefCount > 0 then dec(RfidRefCount);
  if RfidRefCount = 0 then
    FreeLibrary(RfidLibraryHandle);
end;


{ TRFIDReaderThread }

constructor TRFIDReaderThread.Create(ARFIDReader: TRFIDReader);
begin
  FrfidReader := ArfidReader;
  FrfidReader.RegisterEventNotification(TAG_DATA_EVENT, EVENT_HANDLE(FrfidReader.FTagDataEvent));
  inherited Create(False);
  FreeOnTerminate := True;
end;

procedure TRFIDReaderThread.DoOnReadTag;
var tagId: AnsiString;
begin
  tagId := TRFIDReader.GetTagIdA(FpTagData);
  if FrfidReader.TagList.IndexOf(tagId) = -1 then
    FrfidReader.TagList.Append(tagId)
  else if FrfidReader.TagList.Duplicates = dupIgnore then
    Exit;
  if Assigned(FrfidReader.OnReadTag) then
    FrfidReader.OnReadTag(FrfidReader, FpTagData)
  else if Assigned(FrfidReader.OnReadTagCallback) then
    FrfidReader.OnReadTagCallback(FrfidReader, FpTagData);
end;

procedure TRFIDReaderThread.DoOnInventoryStop;
begin
  if Assigned(FrfidReader.OnInventoryStop) then
    FrfidReader.OnInventoryStop(FrfidReader);
end;

procedure TRFIDReaderThread.DoOnDisconnect;
begin
  if Assigned(FrfidReader.OnDisconnect) then
    FrfidReader.OnDisconnect(FrfidReader, @FdisconnectionEventData);
end;

procedure TRFIDReaderThread.DoOnException;
begin
  if Assigned(FrfidReader.OnException) then
    FrfidReader.OnException(FrfidReader, @FreaderExceptionEventData);
end;

procedure TRFIDReaderThread.DoOnTimeout;
begin
  if Assigned(FrfidReader.OnIdle) then
    FrfidReader.OnIdle(FrfidReader);
end;

procedure TRFIDReaderThread.Execute;
var
  hEvents: array[0..4] of HANDLE;
  i: integer;
begin
  hEvents[0] := FrfidReader.FTagDataEvent;
  for i:=1 to high(hEvents) do
    hEvents[i] := CreateEvent(nil, False, False, nil);

  // Register Event Handle for Managing events
  FRFIDReader.RegisterEventNotification(INVENTORY_STOP_EVENT  , EVENT_HANDLE(hEvents[1]));
  FRFIDReader.RegisterEventNotification(ACCESS_STOP_EVENT     , EVENT_HANDLE(hEvents[2]));
  FRFIDReader.RegisterEventNotification(DISCONNECTION_EVENT   , EVENT_HANDLE(hEvents[3]));
  FRFIDReader.RegisterEventNotification(READER_EXCEPTION_EVENT, EVENT_HANDLE(hEvents[4]));

  FpTagData := FrfidReader.AllocateTag;

  while not Terminated do begin
    case WaitForMultipleObjects(Length(hEvents), @hEvents, FALSE, 3000{INFINITE}) of
      WAIT_OBJECT_0:
        begin
        //ResetEvent(FrfidReader.FTagDataEvent);
        while FrfidReader.GetEventData(TAG_DATA_EVENT, FpTagData) do
          Synchronize(@DoOnReadTag);
        end;
      WAIT_OBJECT_0+1, // INVENTORY_STOP_EVENT
      WAIT_OBJECT_0+2: // ACCESS_STOP_EVENT
        begin
        Synchronize(@DoOnInventoryStop);
        break;
        end;
      WAIT_OBJECT_0+3: // DISCONNECTION_EVENT
        begin
        // When this event is signaled the application shall call RFID_Disconnect to clean up the Connection settings in the Dll.
        FrfidReader.GetEventData(DISCONNECTION_EVENT, @FdisconnectionEventData);
        Synchronize(@DoOnDisconnect);
        break;
        end;
      WAIT_OBJECT_0+4: // READER_EXCEPTION_EVENT
        begin
        FrfidReader.GetEventData(READER_EXCEPTION_EVENT, @FreaderExceptionEventData);
        Synchronize(@DoOnException);
        end;
      WAIT_TIMEOUT:
        Synchronize(@DoOnTimeout);
    end;
  end;

  if Assigned(FrfidReader) then begin

    FrfidReader.DeallocateTag(FpTagData);

    if Assigned(FrfidReader.FHandle) then begin
      FrfidReader.DeregisterEventNotification(TAG_DATA_EVENT);
      FrfidReader.DeregisterEventNotification(INVENTORY_STOP_EVENT);
      FrfidReader.DeregisterEventNotification(ACCESS_STOP_EVENT);
      FrfidReader.DeregisterEventNotification(DISCONNECTION_EVENT);
      FrfidReader.DeregisterEventNotification(READER_EXCEPTION_EVENT);
    end;

    for i:=1 to high(hEvents) do
      CloseHandle(hEvents[i]);
  end;
end;


{ TRFIDReader }

constructor TRFIDReader.Create(AHostName: string);
begin
  inherited Create;
  FHostName := AHostName;
  FAutonomousMode := True;
  FTagDataEvent :=  CreateEvent(nil, False, False, nil); // (Attributes,ManualReset,InitialState,Name)
  FTagList := TStringList.Create;
end;

destructor TRFIDReader.Destroy;
begin
  if Assigned(FHandle) then Disconnect;
  CloseHandle(FTagDataEvent);
  FTagList.Free;
  inherited Destroy;
end;

procedure TRFIDReader.CheckStatus(Status: RFID_STATUS);
begin
  if Status <> RFID_API_SUCCESS then
    raise ERFIDError.CreateFmt('RFID Reader Error: (%d) %s', [Status, RFID_GetErrorDescriptionA(Status)]);
end;

procedure TRFIDReader.Connect;
var connectionInfo: CONNECTION_INFO;
begin
  FillByte(connectionInfo, SizeOf(connectionInfo), 0);
  connectionInfo.version := RFID_API3_5_5;
  CheckStatus(RFID_ConnectA(@FHandle, PAnsiChar(FHostName), 0, 0, @connectionInfo));
  FillByte(FReaderCaps, SizeOf(FReaderCaps), 0);
  CheckStatus(RFID_GetReaderCapsA(FHandle, @FReaderCaps));
end;

procedure TRFIDReader.Disconnect;
begin
  RFID_Disconnect(FHandle);
  FHandle := nil;
end;

function TRFIDReader.HasAntenna(antennaID: UINT16): boolean;
var antennaStatus: BYTEBOOL; antennaGain: UINT32;
begin
  CheckStatus(RFID_GetPhysicalAntennaProperties(FHandle, antennaID, @antennaStatus, @antennaGain));
  Result := antennaStatus;
end;

procedure TRFIDReader.SetAntennaConfig(const antennaIDs: array of UINT16;
  ReceiveSensitivityIndex: UINT16; TransmitPowerIndex: UINT16;
  TransmitFrequencyIndex: UINT16);
var i: integer;
begin
  for i:=low(antennaIDs) to high(antennaIDs) do
    CheckStatus(RFID_SetAntennaConfig(FHandle, antennaIDs[i], ReceiveSensitivityIndex, TransmitPowerIndex, TransmitFrequencyIndex));
end;

procedure TRFIDReader.RegisterEventNotification(eventType: RFID_EVENT_TYPE;
  notifyObject: EVENT_HANDLE);
begin
  CheckStatus(RFID_RegisterEventNotification(FHandle, eventType, notifyObject));
end;

procedure TRFIDReader.DeregisterEventNotification(eventType: RFID_EVENT_TYPE);
begin
  CheckStatus(RFID_DeregisterEventNotification(FHandle, eventType));
end;

function TRFIDReader.GetTagStorageSettings: TAG_STORAGE_SETTINGS;
begin
  CheckStatus(RFID_GetTagStorageSettings(FHandle, @Result));
end;

procedure TRFIDReader.SetTagStorageSettings(lpTagStorageSettings: LPTAG_STORAGE_SETTINGS);
var tagStorageSettings: TAG_STORAGE_SETTINGS;
begin
  if lpTagStorageSettings = nil then begin
    tagStorageSettings := GetTagStorageSettings;
    tagStorageSettings.tagFields := ord(FIRST_SEEN_TIME_STAMP) or ord(LAST_SEEN_TIME_STAMP) or
                                    ord(PC) or ord(XPC) or ord(CRC);
    tagStorageSettings.discardTagsOnInventoryStop := True;
    lpTagStorageSettings := @tagStorageSettings;
  end;
  CheckStatus(RFID_SetTagStorageSettings(FHandle, lpTagStorageSettings));
end;

function TRFIDReader.AllocateTag: LPTAG_DATA;
begin
  Result := RFID_AllocateTag(FHandle);
end;

procedure TRFIDReader.DeallocateTag(lpTagData: LPTAG_DATA);
begin
  RFID_DeallocateTag(FHandle, lpTagData);
end;

procedure TRFIDReader.PurgeTags;
begin
  CheckStatus(RFID_PurgeTags(FHandle, nil));
end;

procedure TRFIDReader.SetTriggerInfo(out triggerInfo: TRIGGER_INFO; out tagEventReportInfo: TAG_EVENT_REPORT_INFO);
begin
  if AutonomousMode then
    with triggerInfo do begin
      // This feature can be used to enable reporting tag visibility changes.
      // If a reader supports this feature it reports the Reader Capability TagEventReportingSupported as true.
      tagReportTrigger := 1;
      startTrigger.typ := START_TRIGGER_TYPE_IMMEDIATE;
      stopTrigger.typ := STOP_TRIGGER_TYPE_IMMEDIATE;
      lpTagEventReportInfo := @tagEventReportInfo;

      tagEventReportInfo.reportNewTagEvent := IMMEDIATE; //MODERATED;
      tagEventReportInfo.reportTagBackToVisibilityEvent := MODERATED;
      tagEventReportInfo.reportTagInvisibleEvent := NEVER; //MODERATED;
      tagEventReportInfo.newTagEventModeratedTimeoutMilliseconds := 100;
      tagEventReportInfo.tagBackToVisibilityModeratedTimeoutMilliseconds := 100;
      tagEventReportInfo.tagInvisibleEventModeratedTimeoutMilliseconds := 1000;
    end
  else
    with triggerInfo do begin
      tagReportTrigger := 1; // 0 - Tag reports will be triggered when the Stop-Criteria is met.
      startTrigger.typ := START_TRIGGER_TYPE_IMMEDIATE;
      stopTrigger.typ := STOP_TRIGGER_TYPE_IMMEDIATE;
      lpTagEventReportInfo := nil;
    end;
end;

procedure TRFIDReader.PerformInventory;
var
  triggerInfo: TRIGGER_INFO;
  tagEventReportInfo: TAG_EVENT_REPORT_INFO;
begin
  // Start the Thread for receiving the Events from the Reader
  FrfidReaderThread := TRfidReaderThread.Create(Self);

  SetTriggerInfo(triggerInfo, tagEventReportInfo);

  PerformInventory(@triggerInfo);
end;

procedure TRFIDReader.PerformInventory(lpTriggerInfo: LPTRIGGER_INFO);
begin
  CheckStatus(RFID_PerformInventory(FHandle, nil, nil, lpTriggerInfo, nil));
end;

procedure TRFIDReader.StopInventory;
begin
  CheckStatus(RFID_StopInventory(FHandle));
end;

procedure TRFIDReader.InitializeAccessSequence;
begin
  CheckStatus(RFID_InitializeAccessSequence(FHandle));
end;

function TRFIDReader.AddOperationToAccessSequence(lpOpCodeParams: LPOP_CODE_PARAMS): integer;
var opCodeIndex: UINT32;
begin
  CheckStatus(RFID_AddOperationToAccessSequence(FHandle, lpOpCodeParams, @opCodeIndex));
  Result := opCodeIndex;
end;

procedure TRFIDReader.DeinitializeAccessSequence;
begin
  CheckStatus(RFID_DeinitializeAccessSequence(FHandle));
end;

procedure TRFIDReader.PerformAccessSequence;
var
  triggerInfo: TRIGGER_INFO;
  tagEventReportInfo: TAG_EVENT_REPORT_INFO;
  opCodeParams: OP_CODE_PARAMS;
  readAccessParams: READ_ACCESS_PARAMS;
begin
  // Start the Thread for receiving the Events from the Reader
  FrfidReaderThread := TRfidReaderThread.Create(Self);

  SetTriggerInfo(triggerInfo, tagEventReportInfo);

  opCodeParams.opCode := ACCESS_OPERATION_READ;
  opCodeParams.opParams := @readAccessParams;
  readAccessParams.byteOffset:=0;
  readAccessParams.byteCount:=0;
  readAccessParams.accessPassword:=0;
  readAccessParams.memoryBank:=MEMORY_BANK_TID; //MEMORY_BANK_USER;

  InitializeAccessSequence;
  AddOperationToAccessSequence(@opCodeParams);
  PerformAccessSequence(@triggerInfo);
end;

procedure TRFIDReader.PerformAccessSequence(lpTriggerInfo: LPTRIGGER_INFO);
begin
  CheckStatus(RFID_PerformAccessSequence(FHandle, nil, nil, lpTriggerInfo, nil));
end;

procedure TRFIDReader.StopAccessSequence;
begin
  CheckStatus(RFID_StopAccessSequence(FHandle));
end;

function TRFIDReader.GetEventData(eventType: RFID_EVENT_TYPE;
  pEventData: STRUCT_HANDLE): boolean;
begin
  Result := RFID_GetEventData(FHandle, eventType, pEventData) = RFID_API_SUCCESS;
end;

function TRFIDReader.GetReadTag(lpTagData: LPTAG_DATA): boolean;
begin
  // TAG_DATA:
  //   pTagID: Tag ID, for C1G2 this field refers EPC Data.
  //   tagIDLength: in bytes
  //   tagSeenCount:
  //   tagEvent: UNKNOWN_STATE = 0,
  //             NEW_TAG_VISIBLE = 1,
  //             TAG_NOT_VISIBLE = 2,
  //             TAG_BACK_TO_VISIBILITY = 3,
  //             NONE = 4
  //   opStatus: ACCESS_OPERATION_STATUS
  Result := RFID_GetReadTag(FHandle, lpTagData) = RFID_API_SUCCESS;
end;

function TRFIDReader.Read(lpTagData_in, lpTagData_out: LPTAG_DATA): boolean;
var ReadAccessParams: READ_ACCESS_PARAMS;
begin
  ReadAccessParams.accessPassword:=0;
  ReadAccessParams.memoryBank:=lpTagData_out^.memoryBank;
  ReadAccessParams.byteOffset:=lpTagData_out^.memoryBankDataByteOffset;
  ReadAccessParams.byteCount :=lpTagData_out^.memoryBankDataLength;
  if lpTagData_in <> nil then
    Result := RFID_Read(FHandle, lpTagData_in^.pTagID,  lpTagData_in^.tagIDLength, @ReadAccessParams, nil, nil, lpTagData_out, nil) = RFID_API_SUCCESS;
end;

procedure TRFIDReader.WriteTagId(lpTagData: LPTAG_DATA; const TagId: integer);
var hexTagId, binTagId: AnsiString;
    b: array[0..11] of AnsiChar; // Default 96 bit EPC-Id
begin
  // Gen 2 tags contain four types of memory:
  // - Reserved memory (This memory bank stores the kill password and the access password (each are 32 bits).
  //                    The kill password permanently disables the tag (very rarely used), and the access password is set to lock and unlock the tag’s read/write capabilities.)
  // - EPC memory  (This memory bank stores the EPC code, or the Electronic Product Code. It has a minimum of 96 bits of writable memory.
  //                The EPC memory is what is typically used in most applications if they only need 96 bits of memory.)
  // - TID memory  (This memory is used only to store the unique tag ID number by the manufacturer when the IC is manufactured.
  //                Typically, this memory portion cannot be changed.)
  // - User memory (If the user needs more memory than the EPC section has available, certain ICs have extended user memory which can store more information.
  //                When it comes to user memory, there is no standard in how many bits of memory are writable on each tag. Typically, the extended memory is no more than 512 bits, but there are some high memory tags with up to 4K or 8K bytes of memory.
  //                This is the second writable memory bank for Gen 2 ICs(Integrated Circuit).)
  // EPC memory region contains:
  //   word 0: the CRC-16 (16 bits CRC),
  //   word 1: the PC Bits (Protocol Control word); Bits 0-4: EPC Length (number of words of EPC data that will be backscattered during inventory, 3000h=00110000b=6 words); Bit 5: User Memory Indicator bit (0=the tag does not have User Memory, 1=tag does support User Memory)
  //   word 2: the EPC data backscattered during the inventory process starts at word 2.
  // The total number of words available to store EPC data is dependent on the actual tag chip used to make the Gen2 tag
  hexTagId := Format('%x', [TagId]);
  SetString(binTagId, b, HexToBin(PAnsiChar(hexTagId), PAnsiChar(b), Length(b)));
  WriteTagId(lpTagData, binTagId);
end;

procedure TRFIDReader.WriteTagId(lpTagData: LPTAG_DATA; const TagId: AnsiString);
var wp: WRITE_SPECIFIC_FIELD_ACCESS_PARAMS;
begin
  wp.pWriteData:= @tagId[1];
  wp.writeDataLength := Length(TagId);
  wp.accessPassword := 0;
  CheckStatus(RFID_WriteTagId(FHandle, lpTagData^.pTagID, lpTagData^.tagIDLength, @wp, nil, nil));
end;

class function TRFIDReader.IsTagIdA(lpTagData: LPTAG_DATA): boolean;
var i: integer;
begin
  Result := True;
  for i:=0 to lpTagData^.tagIDLength-1 do
    Result := Result and (lpTagData^.pTagID[i] in [32..127]);
end;

class function TRFIDReader.GetTagIdA(lpTagData: LPTAG_DATA): AnsiString;
begin
  SetString(Result, PAnsiChar(lpTagData^.pTagID), lpTagData^.tagIDLength);
end;

class function TRFIDReader.GetTagIdH(lpTagData: LPTAG_DATA): AnsiString;
begin
  SetLength(Result, 2*lpTagData^.tagIDLength);
  BinToHex(PAnsiChar(lpTagData^.pTagID), PAnsiChar(Result), lpTagData^.tagIDLength);
end;

function TRFIDReader.GetLastErrorInfo: ERROR_INFOA;
begin
  RFID_GetLastErrorInfoA(FHandle, @Result);
end;

end.

